import vmi


def save_docx():
    doc = vmi.docx_document('二维规划报告')
    vmi.docx_section_landscape_A4(doc.sections[0])
    vmi.docx_section_margin_middle(doc.sections[0])
    vmi.docx_section_header(doc.sections[0])

    s_title = vmi.docx_style(doc, '影为标题', 'Normal', size=26, alignment='CENTER')
    s_header = vmi.docx_style(doc, '影为页眉', 'Header', size=10)
    s_body_left = vmi.docx_style(doc, '影为正文', 'Body Text')
    s_body_left_indent = vmi.docx_style(doc, '影为正文首行缩进', 'Body Text', first_line_indent=2)
    s_body_center = vmi.docx_style(doc, '影为正文居中', 'Body Text', alignment='CENTER')
    s_table = vmi.docx_style(doc, '影为表格', 'Table Grid', alignment='CENTER')

    doc.sections[0].header.paragraphs[0].text = '影为医疗科技（上海）有限公司\t\t\t\t{}'.format('[表单号]')
    doc.sections[0].header.paragraphs[0].style = s_header

    def add_row_text(table, texts, styles, merge=None):
        r = len(table.add_row().table.rows) - 1
        if merge is not None:
            table.cell(r, merge[0]).merge(table.cell(r, merge[1]))
        for c in range(min(len(table.columns), len(texts))):
            table.cell(r, c).paragraphs[0].text = texts[c]
            table.cell(r, c).paragraphs[0].style = styles[c]

    doc_table = doc.add_table(1, 4, s_table)
    doc_table.autofit = True
    doc_table.alignment = vmi.docx.enum.table.WD_TABLE_ALIGNMENT.CENTER

    doc_table.cell(0, 0).merge(doc_table.cell(0, 3))
    doc_table.cell(0, 0).paragraphs[0].text = '二维规划报告\n2D Planning Report'
    doc_table.cell(0, 0).paragraphs[0].style = s_title

    add_row_text(doc_table, ['顾客名称\nCustomer name', '',
                             '顾客编号\nCustomer number', ''], [s_body_center] * 4)
    add_row_text(doc_table, ['医师\nPhysician', '',
                             '电邮\nEmail', ''], [s_body_center] * 4)
    add_row_text(doc_table, ['联系人\nContact', '',
                             '电话\nPhone number', ''], [s_body_center] * 4)

    doc.save('C:/Users/YW/Desktop/test_docx.docx')


if __name__ == '__main__':
    save_docx()



import vmi


def read_dicom():
    dcmdir = vmi.askDirectory()  # 用户选择文件夹

    if dcmdir is not None:  # 判断用户选中了有效文件夹并点击了确认
        series_list = vmi.sortSeries(dcmdir)  # 将文件夹及其子目录包含的所有DICOM文件分类到各个系列

        if len(series_list) > 0:  # 判断该文件夹内包含有效的DICOM系列
            series = vmi.askSeries(series_list)  # 用户选择DICOM系列

            if series is not None:  # 判断用户选中了有效系列并点击了确认
                return series.read()  # 读取DICOM系列为图像数据


def view_surface(view, image, iso_value):
    pd = vmi.imIsosurface(image, iso_value)  # 提取CT值为iso_value的等值面，面绘制三维重建
    actor = vmi.PolyActor(view)  # 面绘制显示
    actor.setData(pd)  # 载入三维重建得到的面网格模型
    view.setCamera_FitAll()  # 自动调整视图的视野范围
    return actor


def view_volume(view, image, iso_value):
    volume = vmi.ImageVolume(view)  # 体绘制显示
    volume.setData(image)  # 载入读取到的DICOM图像
    volume.setOpacityScalar({iso_value - 1: 0, iso_value: 1})  # 设置像素和透明度之间的函数关系
    view.setCamera_FitAll()  # 自动调整视图的视野范围
    return view, volume


# 导入Qt用于窗口布局
from PySide2.QtWidgets import QWidget, QHBoxLayout

image = read_dicom()  # 读取DICOM图像

if image is None:  # 图像无效则退出主程序
    vmi.appexit()

# 输入一个CT值用于等值面提取
iso_value = vmi.askInt(-1000, 200, 3000, None, 'HU')
if iso_value is None:
    vmi.appexit()

surface_view = vmi.View()  # 面绘制视图
view_surface(surface_view, image, iso_value)

volume_view = vmi.View()  # 体绘制视图
view_volume(volume_view, image, iso_value)

# 创建窗口，布局视图
widget = QWidget()
layout = QHBoxLayout(widget)
layout.addWidget(surface_view)
layout.addWidget(volume_view)

vmi.appexec(widget)  # 执行主程序
vmi.appexit()  # 退出主程序



# -*- coding: utf-8 -*-

# Resource object code
#
# Created: 周二 9月 10 17:42:43 2019
#      by: The Resource Compiler for PySide2 (Qt v5.13.0)
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore

qt_resource_data = b"\
\x00\x00\x00\x10\
<\
\xb8d\x18\xca\xef\x9c\x95\xcd!\x1c\xbf`\xa1\xbd\xdd\
"

qt_resource_name = b"\
\x00\x03\
\x00\x00x\x83\
\x00q\
\x00r\x00c\
\x00\x02\
\x00\x00\x07\xb2\
\x00t\
\x00r\
\x00\x08\
\x0e8\x05}\
\x00z\
\x00h\x00_\x00C\x00N\x00.\x00q\x00m\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x0c\x00\x02\x00\x00\x00\x01\x00\x00\x00\x03\
\x00\x00\x00\x16\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



import vmi

# 创建视图，设置冠状位相机
view = vmi.View()
view.setCamera_Coronal()

# 创建局部坐标系
cs = vmi.CS4x4(axis0=[1, 0, 0], axis1=[0, 0, 1], axis2=[0, -1, 0])

# 创建文字形状
shape = cs.workplane().text('Hello World', 10, 1).val().wrapped

# 创建场景对象
prop = vmi.PolyActor(view, color=[1, 0.6, 0.6])
prop.setData(shape)

# 重置相机适应场景大小
view.setCamera_FitAll()

# 启动主程序
vmi.appexec(view)

# 退出主程序
vmi.appexit()



from typing import List, Optional, Union

import numpy as np
import vtk
from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtOpenGL import QGLWidget
from PySide2.QtWidgets import *

import vmi

vtk.vtkObject().SetGlobalWarningDisplay(0)


class Mouse:
    """响应鼠标事件的对象"""

    def __init__(self, menu=None):
        i = 0

        def e():
            nonlocal i
            name = '_MouseEvent{}'.format(i)
            if not hasattr(self, name):
                setattr(self, '_MouseEvent{}'.format(i), self.mousePass)
                i += 1
            return getattr(self, name)

        self.mouse = {
            'NoButton': {'Enter': e(), 'Leave': e(), 'Wheel': e(), 'Move': e()},
            'LeftButton': {'Press': e(), 'PressMove': e(), 'PressMoveRelease': e(), 'PressRelease': e()},
            'MidButton': {'Press': e(), 'PressMove': e(), 'PressMoveRelease': e(), 'PressRelease': e()},
            'RightButton': {'Press': e(), 'PressMove': e(), 'PressMoveRelease': e(), 'PressRelease': e()}}

        for b in self.mouse:
            for e in self.mouse[b]:
                self.mouse[b][e] = []

        if menu is not None:
            self.mouse['RightButton']['PressRelease'] = [menu.menuexec]

    @staticmethod
    def mousePass(**kwargs):
        pass

    @staticmethod
    def mouseBlock(**kwargs):
        return True


class Menu:
    """响应菜单事件的对象"""

    def __init__(self):
        self.menu = QMenu()
        self.menu.setTitle('选项')

    def __setstate__(self, state):
        self.__init__()
        self.__dict__.update(state)

    def __getstate__(self):
        state = self.__dict__.copy()
        state = {kw: state[kw] for kw in state if kw not in ['menu']}
        return state

    def menuexec(self, **kwargs):
        """在当前鼠标位置弹出菜单"""
        self.menu.exec_(QCursor.pos())
        return True


class View(QGLWidget, Menu, Mouse):
    """场景视图，在QWidget内显示vtkRenderWindow内容"""

    def __init__(self, parent=None):
        vmi.appViews.append(self)
        QGLWidget.__init__(self, parent)
        Menu.__init__(self)

        self.actions = {'CameraFit': QAction(self.tr('最适 (Fit)')),
                        'CameraReverse': QAction(self.tr('反向 (Reverse)')),
                        'CameraAxial': QAction(self.tr('横断位 (Axial)')),
                        'CameraSagittal': QAction(self.tr('矢状位 (Sagittal)')),
                        'CameraCoronal': QAction(self.tr('冠状位 (Coronal)')),
                        'Snapshot': QAction(self.tr('快照 (Snapshot)')), }

        self.actions['CameraFit'].triggered.connect(self.setCamera_FitAll)
        self.actions['CameraReverse'].triggered.connect(self.setCamera_Reverse)
        self.actions['CameraAxial'].triggered.connect(self.setCamera_Axial)
        self.actions['CameraSagittal'].triggered.connect(self.setCamera_Sagittal)
        self.actions['CameraCoronal'].triggered.connect(self.setCamera_Coronal)
        self.actions['Snapshot'].triggered.connect(self.actionSnapshot)

        self.menu.addAction(self.actions['CameraFit'])
        self.menu.addAction(self.actions['CameraReverse'])
        self.menu.addAction(self.actions['CameraAxial'])
        self.menu.addAction(self.actions['CameraSagittal'])
        self.menu.addAction(self.actions['CameraCoronal'])
        self.menu.addAction(self.actions['Snapshot'])

        Mouse.__init__(self, menu=self)
        self.mouse['LeftButton']['PressMove'] = [self.mouseRotateFocal]
        self.mouse['MidButton']['PressMove'] = [self.mousePan]
        self.mouse['RightButton']['PressMove'] = [self.mouseZoom]
        self.mouse['NoButton']['Wheel'] = [self.mouseRotateLook]

        rect = QGuiApplication.screens()[0].geometry()
        rect = QRect(rect.center() - QPoint(800, 450), QSize(1600, 900))
        self.setGeometry(rect)
        self.setAttribute(Qt.WA_OpaquePaintEvent)
        self.setAttribute(Qt.WA_PaintOnScreen)
        self.setFocusPolicy(Qt.WheelFocus)
        self.setMouseTracking(True)

        # 渲染器
        self._Renderer = vtk.vtkRenderer()
        self._Renderer.SetUseHiddenLineRemoval(1)  # 隐线消除
        self._Renderer.SetUseFXAA(False)  # 快速近似抗锯齿
        self._Renderer.SetBackground(1, 1, 1)

        # 渲染窗口
        self._RenderWindow = vtk.vtkRenderWindow()
        self._RenderWindow.SetWindowInfo(str(int(self.winId())))
        self._RenderWindow.AddRenderer(self._Renderer)

        # 交互器
        self._Interactor = vtk.vtkGenericRenderWindowInteractor()
        self._Interactor.SetRenderWindow(self._RenderWindow)
        self._Interactor.Initialize()

        # 方位标
        self._OrientationMarker = _init_orientation_marker(self._Interactor)

        # 照相机
        self._Camera = self.renderer().GetActiveCamera()
        self._BindCamera = [self]
        self.camera().SetParallelProjection(True)  # 平行投影

        # 拾取器
        self._Picker = vtk.vtkPicker()
        self._PickerProp = vtk.vtkPropPicker()
        self._PickerPoint = vtk.vtkPointPicker()
        self._PickerCell = vtk.vtkCellPicker()
        self._Picker.SetTolerance(0.005)
        self._PickerPoint.SetTolerance(0.005)
        self._PickerCell.SetTolerance(0.005)

        # 刷新计时器
        self._Timer = QTimer(self)
        self._Timer.timeout.connect(self._timeout)
        self._Timer.start(10)

        self._Time = QTime()
        self._Time.start()
        self._FPS = self._UpdateSum = 0

        # 创建一个隐藏控件，当销毁时调用清理vtk元素
        self._Hidden = QWidget(self)
        self._Hidden.hide()
        self._Hidden.destroyed.connect(self.delete)

        # 更新请求
        self._UpdateAtOnce: bool = False

        # 鼠标位置
        self._DblClick = False
        self._MousePosLast: QPoint = QPoint(0, 0)
        self._MousePos: QPoint = QPoint(0, 0)

        # 鼠标正在交互的对象
        self._PProp: Optional[vtk.vtkProp] = None
        self._PView: Optional[View] = None

        # 鼠标按键状态
        self._ButtonPress = self._ButtonPressMove = 'NoButton'

        self.setCamera_Axial()

    def __setstate__(self, state):
        self.__init__()

        for kw in state['vtk']:
            vmi.vtkInstance_sets(getattr(self, kw), state['vtk'][kw])

        for kw in state['set']:
            if kw not in ['_Visible']:
                getattr(self, kw.replace('_', 'set', 1))(state['set'][kw])

        QTimer.singleShot(10, self.show if state['set']['_Visible'] else self.hide)

        del state['vtk']
        del state['set']
        self.__dict__.update(state)

    def __getstate__(self):
        state = self.__dict__.copy()
        state = {kw: state[kw] for kw in state if kw not in
                 ['menu', 'menuCamera', 'actions', '_OrientationMarker', '_Timer', '_Hidden', 'aboutToClose',
                  '__METAOBJECT__']}

        state_set = {'_Enabled': self.isEnabled(), '_Visible': self.isVisible(), '_WindowTitle': self.windowTitle()}
        state = {kw: state[kw] for kw in state if kw not in state_set}

        state_vtk = {}
        for kw in state:
            if isinstance(state[kw], vtk.vtkObject):
                if not hasattr(vmi, 'pickle_' + state[kw].__class__.__name__):
                    state_vtk[kw] = vmi.vtkInstance_gets(state[kw])
        state = {kw: state[kw] for kw in state if kw not in state_vtk}

        state['set'] = state_set
        state['vtk'] = state_vtk
        return state

    def closeEvent(self, ev):
        self.delete()

    def delete(self):
        """停止场景刷新"""
        self._Timer.stop()
        self._RenderWindow.Finalize()

    def mouseExec(self, cats, button, event, picked=None, **kwargs) -> None:
        """
        顺序响应鼠标事件

        :param cats: 响应者列表
        :param button: 鼠标按键
        :param event: 事件内容
        :param picked: 鼠标正在交互的对象
        :param kwargs: 其它参数
        :return: None
        """
        if picked is None:
            if self._PView:
                picked = self._PView
            if self._PProp:
                picked = self._PProp

        for cat in [cat for cat in cats if isinstance(cat, Mouse)]:
            for mbe in cat.mouse[button][event]:
                if mbe(picked=picked, **kwargs):
                    return

    def entered(self):
        """返回鼠标正在交互的对象"""
        return self._PProp if self._PProp else (self._PView if self._PView else self)

    def enterEvent(self, ev):
        self._PView = self
        self._PProp = self.pickerExec(pos=ev.pos(), picker=self._PickerProp)[0]
        self.mouseExec([self._PView, self._PProp], 'NoButton', 'Enter')

    def leaveEvent(self, ev):
        self.mouseExec([self._PProp, self._PView], 'NoButton', 'Leave')
        self._PView = self._PProp = None
        self._ButtonPress = self._ButtonPressMove = 'NoButton'

    def mouseMoveEvent(self, ev):
        self._MousePosLast, self._MousePos = self._MousePos, ev.pos()

        if self._ButtonPress == 'NoButton':
            prop = self.pickerExec(pos=ev.pos(), picker=self._PickerProp)[0]
            last_prop, self._PProp = self._PProp, prop

            if not last_prop and not prop:
                self.mouseExec([self._PView], 'NoButton', 'Move')
            elif not last_prop and prop:
                self.mouseExec([prop], 'NoButton', 'Enter', prop)
            elif last_prop and not prop:
                self.mouseExec([last_prop], 'NoButton', 'Leave', last_prop)
            elif last_prop and prop and last_prop is prop:
                self.mouseExec([self._PProp, self._PView], 'NoButton', 'Move')
            elif last_prop and prop and last_prop is not prop:
                self.mouseExec([last_prop], 'NoButton', 'Leave', last_prop)
                self.mouseExec([prop], 'NoButton', 'Enter', prop)
        elif self._MousePosLast != self._MousePos:
            self._ButtonPressMove = self._ButtonPress
            self.mouseExec([self._PProp, self._PView], self._ButtonPressMove, 'PressMove')
        ev.accept()

    def mousePressEvent(self, ev):
        self._MousePosLast, self._MousePos = self._MousePos, ev.pos()

        if self._ButtonPress == 'NoButton':
            b = ev.button().name.decode()
            if b in ['LeftButton', 'MidButton', 'RightButton']:
                self._ButtonPress = b
                self._DblClick = ev.type() == QEvent.Type.MouseButtonDblClick
                self.mouseExec([self._PProp, self._PView], self._ButtonPress, 'Press', double=self._DblClick)
        ev.accept()

    def mouseReleaseEvent(self, ev):
        self._MousePosLast, self._MousePos = self._MousePos, ev.pos()

        if self._ButtonPressMove == ev.button().name.decode():
            last_button = self._ButtonPress
            self._ButtonPress = self._ButtonPressMove = 'NoButton'
            self.mouseExec([self._PProp, self._PView], last_button, 'PressMoveRelease')
        elif self._ButtonPress == ev.button().name.decode():
            last_button, last_double = self._ButtonPress, self._DblClick
            self._ButtonPress = self._ButtonPressMove = 'NoButton'
            self._DblClick = False
            self.mouseExec([self._PProp, self._PView], last_button, 'PressRelease', double=last_double)

    def mouseRotateFocal(self, **kwargs):
        """响应鼠标移动，观察者绕焦点旋转"""
        dx, dy = self.pickVt_Display()
        rx, ry = -200 * dx / self.height(), 200 * dy / self.width()
        self.camera().Azimuth(rx)
        self.camera().Elevation(ry)
        self.camera().OrthogonalizeViewUp()
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def mouseRotateLook(self, **kwargs):
        """响应鼠标移动，观察者绕视线旋转"""
        self.camera().Roll(0.5 if kwargs['delta'] > 0 else -0.5)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def mousePan(self, **kwargs):
        """响应鼠标移动，观察者平移"""
        self.updateInTime()
        over = self.pickVt_FocalPlane()
        fp, cp = self.cameraPt_Focal(), self.cameraPt_Lens()
        fp -= over
        cp -= over
        self.camera().SetFocalPoint(fp)
        self.camera().SetPosition(cp)

        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()
            if i._Interactor.GetLightFollowCamera():
                i._Renderer.UpdateLightsGeometryToFollowCamera()

    def mouseZoom(self, **kwargs):
        """响应鼠标移动，观察者放缩"""
        self.updateInTime()
        c = self._Renderer.GetCenter()
        _, dy = self.pickVt_Display()
        factor = 1.1 ** (10 * -dy / c[1])

        if self.camera().GetParallelProjection():
            self.camera().SetParallelScale(self.camera().GetParallelScale() / factor)
        else:
            self.camera().Dolly(factor)

        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()
            if self._Interactor.GetLightFollowCamera():
                self._Renderer.UpdateLightsGeometryToFollowCamera()

    def pickPt_Display(self) -> np.ndarray:
        """
        拾取鼠标指针所在的屏幕坐标

        :return: [x, y]
        """
        return np.array([self._MousePos.x(), self._MousePos.y()])

    def pickPt_FocalPlane(self, pos: Union[QPoint, List[float]] = None) -> np.ndarray:
        """
        拾取鼠标指针在焦平面上的点

        :param pos: 指针位置，默认事件发生位置
        :return: [x, y, z]
        """
        if pos is None:
            pos = self._MousePos
        if not isinstance(pos, QPoint):
            pos = QPoint(pos[0], pos[1])

        t = [0, 0, 0, 0]
        vtk.vtkInteractorObserver.ComputeDisplayToWorld(self._Renderer, pos.x(), self.height() - pos.y(), 0, t)
        pt = np.array([_ / t[3] for _ in t[:3]])

        fp, look = self.cameraPt_Focal(), self.cameraVt_Look()
        pt = vmi.ptOnPlane(pt, fp, look)
        return pt

    def pickPt_Prop(self, pos: Optional[QPoint] = None) -> vtk.vtkProp:
        """
        拾取鼠标指针所在的物体

        :param pos: 指针位置，默认事件发生位置
        :return: prop
        """
        if pos is None:
            pos = self._MousePos
        return self.pickerExec(pos=pos, picker=self._PickerProp)[0]

    def pickPt_Cell(self, pos: Optional[QPoint] = None) -> np.ndarray:
        if pos is None:
            pos = self._MousePos
        return self.pickerExec(pos=pos, picker=self._PickerCell)[1]

    def pickVt_Display(self) -> np.ndarray:
        """
        拾取鼠标指针最近一次移动的屏幕位移

        :return: [dx, dy]
        """
        return np.array([self._MousePos.x() - self._MousePosLast.x(), self._MousePos.y() - self._MousePosLast.y()])

    def pickVt_FocalPlane(self) -> np.ndarray:
        """
        拾取鼠标指针最近一次移动的焦平面位移

        :return: [dx, dy, dz]
        """
        pt = np.array(self.pickPt_FocalPlane(self._MousePos))
        ptLast = np.array(self.pickPt_FocalPlane(self._MousePosLast))
        vt = pt - ptLast
        return vt

    def pickPt_DisplayInv(self, pt):
        """
        拾取世界坐标点pt对应的屏幕坐标

        :param pt: 世界坐标点
        :return: [x, y]
        """
        t = [0, 0, 0]
        vtk.vtkInteractorObserver.ComputeWorldToDisplay(self._Renderer, pt[0], pt[1], pt[2], t)
        return np.array([t[0], self.height() - t[1]])

    def paintEngine(self):
        return None

    def paintEvent(self, ev):
        self._Interactor.Render()

    # 可能会导致app.processEvents崩溃

    def pickerExec(self, picker: vtk.vtkPicker, pos: Optional[QPoint] = None) -> [vtk.vtkProp, np.ndarray,
                                                                                  vtk.vtkPicker]:
        pos = self._MousePos if pos is None else pos

        picker.Pick(pos.x(), self.height() - pos.y(), 0, self._Renderer)
        prop = picker.GetViewProp()
        prop = prop._Prop if prop else None
        return prop, np.array(picker.GetPickPosition()), picker

    def resizeEvent(self, ev):
        w = self.width()
        h = self.height()
        vtk.vtkRenderWindow.SetSize(self._RenderWindow, w, h)
        self._Interactor.SetSize(w, h)
        self._Interactor.ConfigureEvent()
        self.update()

        props: vtk.vtkPropCollection = self._Renderer.GetViewProps()
        props.InitTraversal()
        prop = props.GetNextProp()

        while prop:
            if hasattr(prop, '_Prop') and hasattr(prop._Prop, 'resize'):
                prop._Prop.resize()
            prop = props.GetNextProp()

        self.updateInTime()

    def renderer(self) -> vtk.vtkRenderer:
        """返回渲染器vtkRenderer"""
        return self._Renderer

    def camera(self) -> vtk.vtkCamera:
        """返回观察者vtkCamera"""
        return self._Camera

    def bindCamera(self, others):
        """关联相机，使视图采用other视图的视角，other = self 关联自身，other = None 重新关联当前对象"""
        binds = others.copy()
        if self not in binds:
            binds.append(self)
        for i in binds:
            i._BindCamera = binds.copy()
        for i in self._BindCamera:
            i.camera().DeepCopy(self._BindCamera[0].camera())
            i.updateInTime()

    def cameraPt_Focal(self) -> np.ndarray:
        """返回观察焦点坐标"""
        return np.array(self.camera().GetFocalPoint())

    def cameraPt_Lens(self) -> np.ndarray:
        """返回观察者位置坐标"""
        return np.array(self.camera().GetPosition())

    def cameraVt_Up(self) -> np.ndarray:
        """返回观察者上向向量"""
        up = np.array(self.camera().GetViewUp())
        up /= np.linalg.norm(up)
        return up

    def cameraVt_Look(self) -> np.ndarray:
        """返回观察者视向向量"""
        look = self.cameraPt_Focal() - self.cameraPt_Lens()
        look /= np.linalg.norm(look)
        return look

    def cameraVt_Right(self) -> np.ndarray:
        """返回观察者右向向量"""
        right = np.cross(self.cameraVt_Look(), self.cameraVt_Up())
        right /= np.linalg.norm(right)
        return right

    def cameraCS(self) -> vmi.CS4x4:
        """返回观察坐标系

        原点是视图左上角[0, 0]在焦平面上的拾取点

        坐标轴分别是[right, -up, look]向量"""
        cs = vmi.CS4x4()
        cs.setOrigin(self.pickPt_FocalPlane(QPoint(0, 0)))
        cs.setAxis(0, self.cameraVt_Right())
        cs.setAxis(1, -self.cameraVt_Up())
        cs.setAxis(2, self.cameraVt_Look())
        return cs

    def setCamera_FPlane(self, cs: vmi.CS4x4, height: float) -> None:
        """
        设置视图的观察坐标系

        :param cs: 观察坐标系
        :param height: 视图高度对应场景中的世界尺寸
        :return: None
        """
        width = height * self.width() / self.height()
        fp = cs.mpt([0.5 * width, 0.5 * height, 0])
        b6 = self._Renderer.ComputeVisiblePropBounds()
        d = ((b6[1] - b6[0]) ** 2 + (b6[3] - b6[2]) ** 2 + (b6[5] - b6[4]) ** 2) ** 0.5
        cp = cs.mpt([0.5 * width, 0.5 * height, -d])

        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(fp)
        self.camera().SetViewUp(list(-cs.axis(1)))
        self.camera().SetParallelScale(0.5 * height)

        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_Focal(self, focal=Union[np.ndarray, List[float]]) -> None:
        """设置观察焦点，镜头位置随之平移"""
        vl = self.cameraVt_Look()
        b = np.array(self._Renderer.ComputeVisiblePropBounds())
        dxyz = np.array([b[1] - b[0], b[3] - b[2], b[5] - b[4]])
        diagonal = np.linalg.norm(dxyz)
        cp = focal - diagonal * vl

        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(focal)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_FitAll(self) -> None:
        """设置观察范围适应全部场景可见物体"""
        b = np.array(self._Renderer.ComputeVisiblePropBounds())
        dxyz = np.array([b[1] - b[0], b[3] - b[2], b[5] - b[4]])
        self._Renderer.ResetCamera(b)

        vr, vu, vl = self.cameraVt_Right(), self.cameraVt_Up(), self.cameraVt_Look()
        dr = np.abs(vr / np.linalg.norm(vr))
        dr = np.dot(dr, dxyz)
        dr = dr if dr != 0 else 1
        du = np.abs(vu / np.linalg.norm(vu))
        du = np.dot(du, dxyz)
        du = du if du != 0 else 1
        scale = 0.5 * du
        if du / dr < self.height() / self.width():
            scale = 0.5 * dr * self.height() / self.width()

        focal = b[::2] + 0.5 * dxyz
        diagonal = np.linalg.norm(dxyz)
        cp = focal - diagonal * vl
        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(focal)
        self.camera().SetParallelScale(scale)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_Reverse(self) -> None:
        """设置观察者反向，镜头位置切换到焦点的对侧"""
        fp, cp = self.cameraPt_Focal(), self.cameraPt_Lens()
        for i in range(3):
            cp[i] = 2 * fp[i] - cp[i]
        self.camera().SetPosition(cp)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_Axial(self) -> None:
        """设置观察者正视横断面"""
        vu = [np.array([0, 0, 1]), np.array([0, 0, 1]), np.array([0, -1, 0])][2]
        vr = [np.array([0, -1, 0]), np.array([1, 0, 0]), np.array([1, 0, 0])][2]
        vl = np.cross(vu, vr)

        fp = self.cameraPt_Focal()
        d = self.camera().GetDistance()
        cp = fp - d * vl

        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(fp)
        self.camera().SetViewUp(vu)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_Sagittal(self) -> None:
        """设置观察者正视矢状面"""
        vu = [np.array([0, 0, 1]), np.array([0, 0, 1]), np.array([0, -1, 0])][0]
        vr = [np.array([0, -1, 0]), np.array([1, 0, 0]), np.array([1, 0, 0])][0]
        vl = np.cross(vu, vr)

        fp = self.cameraPt_Focal()
        d = self.camera().GetDistance()
        cp = fp - d * vl

        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(fp)
        self.camera().SetViewUp(vu)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCamera_Coronal(self) -> None:
        """设置观察者正视冠状面"""
        vu = [np.array([0, 0, 1]), np.array([0, 0, 1]), np.array([0, -1, 0])][1]
        vr = [np.array([0, -1, 0]), np.array([1, 0, 0]), np.array([1, 0, 0])][1]
        vl = np.cross(vu, vr)

        fp = self.cameraPt_Focal()
        d = self.camera().GetDistance()
        cp = fp - d * vl

        self.camera().SetPosition(cp)
        self.camera().SetFocalPoint(fp)
        self.camera().SetViewUp(vu)
        for i in self._BindCamera:
            i.camera().DeepCopy(self.camera())
            i.updateInTime()

    def setCornerVisible_LeftBottom(self, visible: bool):
        """显示或隐藏左下角方位标"""
        self._OrientationMarker.GetOrientationMarker().SetVisibility(visible)
        self.updateInTime()

    def sizeHint(self):
        return QSize(1600, 900)

    def snapshot(self) -> QImage:
        """快照，保存视图画面为QImage"""
        vmi.appupdate()
        return self.windowHandle().screen().grabWindow(self.winId()).toImage()

    def actionSnapshot(self):
        self.snapshot_PNG()

    def snapshot_PNG(self, file=None) -> None:
        """
        快照，保存视图画面为png图像文件

        :param file: png文件路径
        :return: None
        """
        if file is None:
            file = vmi.askSaveFile(self.tr('快照 (Snapshot)'), '*.png')
        if file is not None:
            if not file.endswith('.png'):
                file += '.png'
            self.snapshot().save(file)

    def _timeout(self):
        if self._UpdateAtOnce:
            self.updateAtOnce()
        self._UpdateSum += 1

        if self._Time.elapsed() > 0.5e3:
            self._FPS = 1000 * self._UpdateSum / self._Time.elapsed()
            self.setWindowTitle(QApplication.applicationName() + ' FPS: {:.0f}'.format(self._FPS))
            self._Time.restart()
            self._UpdateSum = 0

    def updateInTime(self) -> None:
        """及时刷新视图"""
        self._UpdateAtOnce = True

    def updateAtOnce(self) -> None:
        """立即刷新视图"""
        if self._Timer.isActive():
            self._UpdateAtOnce = False
            self._Renderer.ResetCameraClippingRange()
            self._RenderWindow.Render()

    def wheelEvent(self, ev):
        d = int(ev.delta() / 120)
        self.mouseExec([self.entered()], 'NoButton', 'Wheel', delta=d)
        ev.accept()


def _init_orientation_marker(i):
    """
    :param i: vtk.vtkRenderWindowInteractor
    :return: vtk.vtkOrientationMarkerWidget
    """

    vtk.vtkMapper.SetResolveCoincidentTopologyToPolygonOffset()

    cube = vtk.vtkAnnotatedCubeActor()
    cube.SetFaceTextScale(0.65)
    cube.SetXPlusFaceText("L")
    cube.SetXMinusFaceText("R")
    cube.SetYPlusFaceText("P")
    cube.SetYMinusFaceText("A")
    cube.SetZPlusFaceText("S")
    cube.SetZMinusFaceText("I")

    prop = cube.GetCubeProperty()
    prop.SetColor(0.75, 0.75, 1)

    prop = cube.GetTextEdgesProperty()
    prop.SetLineWidth(1)
    prop.SetDiffuse(0)
    prop.SetAmbient(1)
    prop.SetColor(0, 0, 0)

    prop = cube.GetXPlusFaceProperty()
    prop.SetColor(1, 0, 0)
    prop.SetInterpolationToFlat()
    prop = cube.GetXMinusFaceProperty()
    prop.SetColor(1, 0, 0)
    prop.SetInterpolationToFlat()
    prop = cube.GetYPlusFaceProperty()
    prop.SetColor(0, 1, 0)
    prop.SetInterpolationToFlat()
    prop = cube.GetYMinusFaceProperty()
    prop.SetColor(0, 1, 0)
    prop.SetInterpolationToFlat()
    prop = cube.GetZPlusFaceProperty()
    prop.SetColor(0, 0, 1)
    prop.SetInterpolationToFlat()
    prop = cube.GetZMinusFaceProperty()
    prop.SetColor(0, 0, 1)
    prop.SetInterpolationToFlat()

    axes = vtk.vtkAxesActor()

    axes.SetShaftTypeToCylinder()
    axes.SetXAxisLabelText("X")
    axes.SetYAxisLabelText("Y")
    axes.SetZAxisLabelText("Z")

    axes.SetTotalLength(1.5, 1.5, 1.5)
    axes.SetCylinderRadius(0.500 * axes.GetCylinderRadius())
    axes.SetConeRadius(1.025 * axes.GetConeRadius())
    axes.SetSphereRadius(1.500 * axes.GetSphereRadius())

    prop = axes.GetXAxisCaptionActor2D().GetCaptionTextProperty()
    prop.BoldOn()
    prop.ItalicOn()
    prop.ShadowOn()
    prop.SetFontFamilyToArial()
    prop.SetColor(1, 0, 0)

    prop = axes.GetYAxisCaptionActor2D().GetCaptionTextProperty()
    prop.BoldOn()
    prop.ItalicOn()
    prop.ShadowOn()
    prop.SetFontFamilyToArial()
    prop.SetColor(0, 1, 0)

    prop = axes.GetZAxisCaptionActor2D().GetCaptionTextProperty()
    prop.BoldOn()
    prop.ItalicOn()
    prop.ShadowOn()
    prop.SetFontFamilyToArial()
    prop.SetColor(0, 0, 1)

    assembly = vtk.vtkPropAssembly()
    assembly.AddPart(axes)
    assembly.AddPart(cube)

    ort = vtk.vtkOrientationMarkerWidget()
    ort.SetOrientationMarker(assembly)
    ort.SetViewport(0.0, 0.0, 0.2, 0.2)
    ort.SetInteractor(i)
    ort.SetEnabled(1)
    ort.SetInteractive(0)
    ort.On()

    return ort



import pathlib
import time
from typing import List

import numpy as np
import pydicom
import vtk
from PySide2.QtGui import *
from numba import jit

import vmi


def LeftButtonPress(**kwargs):
    if kwargs['picked'] is view:
        if local_roi_box.text() == '请框选目标区域':
            pt = view.pickPt_FocalPlane()
            global local_roi_pts
            local_roi_pts = [pt.copy(), pt.copy()]
            update_local_roi()


def LeftButtonPressMove(**kwargs):
    global spcut_center, pelvis_cs
    if kwargs['picked'] is view:
        if local_roi_box.text() == '请框选目标区域':
            pt = view.pickPt_Cell()
            global local_roi_pts
            local_roi_pts[-1] = pt.copy()
            update_local_roi()
        else:
            view.mouseRotateFocal(**kwargs)


def LeftButtonPressMoveRelease(**kwargs):
    if kwargs['picked'] is view:
        if local_roi_box.text() == '请框选目标区域':
            local_roi_box.draw_text('请耐心等待')
            update_local_solid()
            global local_roi_pts
            local_roi_pts.clear()
            update_local_roi()
            local_roi_box.draw_text('捧一下')


def LeftButtonPressRelease(**kwargs):
    if kwargs['picked'] is local_roi_box:
        if local_roi_box.text() in ['捧一下', '请耐心等待']:
            local_roi_box.draw_text('请框选目标区域')
        elif local_roi_box.text() == '请框选目标区域':
            local_roi_box.draw_text('捧一下')
    elif kwargs['picked'] is view:
        if local_roi_box.text() == '请框选目标区域':
            local_roi_box.draw_text('请耐心等待')
            update_local_solid()
            global local_roi_pts
            local_roi_pts.clear()
            update_local_roi()
            local_roi_box.draw_text('捧一下')


def init_view():
    global target
    target = vmi.pdOpenFile_STL()
    if target is None:
        vmi.appexit()

    global view
    view = vmi.View()
    view.setCamera_Coronal()
    view.mouse['LeftButton']['Press'] = [LeftButtonPress]
    view.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
    view.mouse['LeftButton']['PressMoveRelease'] = [LeftButtonPressMoveRelease]

    global target_prop
    target_prop = vmi.PolyActor(view, color=[0.8, 0.8, 0.8], repres='wireframe')
    target_prop.setData(target)
    view.setCamera_FitAll()

    global local_roi_box
    local_roi_box = vmi.TextBox(view, text='捧一下', size=[0.2, 0.04], pos=[0, 0.1], anchor=[0, 0], pickable=True)
    local_roi_box.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]

    global local_roi_prop
    local_roi_prop = vmi.PolyActor(view, color=[1, 0.6, 0.6], line_width=3, always_on_top=True)

    global local_solid_target_prop, local_solid_match_prop
    local_solid_target_prop = vmi.PolyActor(view, color=[1, 1, 0.6])
    local_solid_match_prop = vmi.PolyActor(view, color=[0.6, 0.8, 1], opacity=0.5)

    global output_solid_target, output_solid_match
    output_solid_target, output_solid_match = None, None

    global output_solid_target_prop, output_solid_match_prop
    output_solid_target_prop = vmi.PolyActor(view, color=[1, 1, 0.4])
    output_solid_match_prop = vmi.PolyActor(view, color=[0.4, 0.6, 1])


def update_local_roi():
    global local_roi_pts
    if len(local_roi_pts) and np.linalg.norm(local_roi_pts[0] - local_roi_pts[1]):
        global target, output_solid_target_prop, local_cs, local_max_z
        local_cs = view.cameraCS()
        if output_solid_target is None:
            local_max_z = np.linalg.norm(vmi.pdSize(target))
            local_center = vmi.pdCenter(target) - 0.5 * local_max_z * local_cs.axis(2)
        else:
            local_max_z = np.linalg.norm(vmi.pdSize(output_solid_target_prop.data()))
            local_center = vmi.pdCenter(output_solid_target_prop.data()) - 0.5 * local_max_z * local_cs.axis(2)

        local_center = vmi.ptOnPlane(local_roi_pts[0], local_center, local_cs.axis(2))
        local_cs.setOrigin(local_roi_pts[0])

        global local_rect_xr, local_rect_yr
        vt = local_roi_pts[0] - local_roi_pts[1]
        vt = local_cs.inv().mvt(vt)
        local_rect_xr, local_rect_yr = np.linalg.norm(vt[0]), np.linalg.norm(vt[1])
        local_roi_wire = local_cs.workplane().rect(local_rect_xr * 2, local_rect_yr * 2).val().wrapped
        local_roi_prop.setData(local_roi_wire)

        local_cs.setOrigin(local_center)
    else:
        local_roi_prop.setData(None)


def update_local_solid():
    global target, local_cs, local_max_z
    res = vmi.askFloat(0.1, 1, 5, 1, 0.1, '精度', 'mm', '精度')
    if res is None:
        vmi.askInfo('已取消')
        return
    smooth = vmi.askInt(0, 0, 10, 1, '光顺指数', None, '光顺指数')
    if smooth is None:
        vmi.askInfo('已取消')
        return
    smooth = res / smooth if smooth else 0
    ok = vmi.askYesNo('确定执行？')
    if not ok:
        vmi.askInfo('已取消')
        return

    xn, yn = int(np.ceil(local_rect_xr / res)) + 1, int(np.ceil(local_rect_yr / res)) + 1
    local_solid_target = []
    local_solid_match = vmi.ccMatchSolid_Rect(
        target, local_cs, xn, yn, res, res, local_max_z, smooth, local_solid_target)
    local_solid_target = local_solid_target[0]

    global output_solid_target, output_solid_match
    if output_solid_target is None:
        output_solid_target = local_solid_target
        output_solid_target_prop.setData(output_solid_target)
        local_solid_match_prop.setData(local_solid_match)
    else:
        output_solid_target = vmi.ccBoolean_Difference([output_solid_target, local_solid_match])
        output_solid_target_prop.setData(output_solid_target)
        # local_solid_target_prop.setData(local_solid_target)
        local_solid_match_prop.setData(local_solid_match)


if __name__ == '__main__':
    vmi.app.setApplicationName('StraightMan')
    init_view()

    global view
    vmi.appexec(view)
    vmi.appexit()



import os
import pathlib
import time

import gridfs
import pymongo
import requests
from PySide2.QtWidgets import *

import vmi
import vmi.application.skyward

case_db = vmi.application.skyward.case_db
order_db = vmi.application.skyward.order_db
user_db = vmi.application.skyward.user_db
role_customer_db = vmi.application.skyward.role_customer_db
role_sales_worker_db = vmi.application.skyward.role_sales_worker_db
role_product_designer_db = vmi.application.skyward.role_product_designer_db
role_quality_inspector_db = vmi.application.skyward.role_quality_inspector_db
dicom_fs = vmi.application.skyward.dicom_fs
vti_fs = vmi.application.skyward.vti_fs
vtp_fs = vmi.application.skyward.vtp_fs
stl_fs = vmi.application.skyward.stl_fs


def tray_icon_activated(reason: QSystemTrayIcon.ActivationReason):
    if reason is QSystemTrayIcon.Trigger:
        print(reason)


def download_order_files(order_uid: str):
    p = vmi.askDirectory('设计文件夹')
    if not p:
        return

    case = case_db.find_one({'order_uid': order_uid})

    for f in case['order_files']:
        if f.endswith('stl'):
            if not stl_fs.find_one({'_id': case[f]}):
                vmi.askInfo('错误：设计文件不存在 {}'.format(f))
                return

    p = pathlib.Path(p) / (order_uid + ' ' + time.strftime('%Y-%m-%d-%H-%M-%S', time.localtime()))
    os.mkdir(p)

    for f in case['order_files']:
        if f.endswith('stl'):
            b = stl_fs.find_one({'_id': case[f]}).read()
            (pathlib.Path(p) / (f + '.stl')).write_bytes(b)

    vmi.askInfo('下载完成')


def find_order_released():
    combo = QComboBox()
    for case in case_db.find():
        if case['customer_confirmed']:
            if case['order_released'] and not case['order_inspected'] and not case['order_completed']:
                t = case['last_update_time'][:14]
                t = [t[:4], t[4:6], t[6:8], t[8:10], t[10:12], t[12:14]]
                t = '-'.join(t)
                combo.addItem(case['order_uid'] + ' ' + t, case['order_uid'])

    if not combo.count():
        vmi.askInfo('无待检生产单')
        return

    sizep = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
    dialog = QDialog()
    dialog.setSizePolicy(sizep)
    dialog.setWindowTitle('选择待检生产单')
    dialog.setLayout(QVBoxLayout())
    dialog.setMinimumWidth(200)
    dialog.layout().addWidget(combo)

    ok_cancel = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, dialog)
    ok_cancel.setSizePolicy(sizep)
    ok_cancel.accepted.connect(dialog.accept)
    ok_cancel.rejected.connect(dialog.reject)
    dialog.layout().addWidget(ok_cancel)

    if dialog.exec_() != QDialog.Accepted:
        return

    r = vmi.askButtons(['下载设计文件', '接受生产单', '驳回生产单'])
    if r == '下载设计文件':
        user = vmi.application.skyward.ask_user()
        if not user:
            return False

        role_quality_inspector_id = user['role_quality_inspector_id']
        if not role_quality_inspector_db.find_one({'_id': role_quality_inspector_id}):
            vmi.askInfo('错误：只有质量检验员可以下载设计文件')
            return False

        download_order_files(combo.currentData())
    elif r == '接受生产单':
        user = vmi.application.skyward.ask_user()
        if not user:
            return False

        role_quality_inspector_id = user['role_quality_inspector_id']
        if not role_quality_inspector_db.find_one({'_id': role_quality_inspector_id}):
            vmi.askInfo('错误：只有质量检验员可以接受生产单')
            return False

        case_set = {'role_quality_inspector_id': role_quality_inspector_id,
                    'order_released': True,
                    'order_inspected': True,
                    'order_completed': False}

        try:
            find = case_db.find_one({'order_uid': combo.currentData()})
            case_uid = vmi.convert_base(round(time.time() * 100))
            response = requests.post(url='http://med-3d.top:2334/sys/caseUpdate',
                                     headers={'_id': str(find['_id']), 'case_uid': case_uid})
            if response.text == 'true':
                case_db.find_one_and_update({'_id': find['_id']}, {'$set': case_set})
                vmi.askInfo('同步完成')
                return True
            else:
                vmi.askInfo('错误：' + response.text)
        except Exception as e:
            vmi.askInfo('错误：' + str(e))
    elif r == '驳回生产单':
        user = vmi.application.skyward.ask_user()
        if not user:
            return False

        role_quality_inspector_id = user['role_quality_inspector_id']
        if not role_quality_inspector_db.find_one({'_id': role_quality_inspector_id}):
            vmi.askInfo('错误：只有质量检验员可以驳回生产单')
            return False

        case_set = {'role_quality_inspector_id': role_quality_inspector_id,
                    'order_released': False,
                    'order_inspected': False,
                    'order_completed': False}

        try:
            find = case_db.find_one({'order_uid': combo.currentData()})
            case_uid = vmi.convert_base(round(time.time() * 100))
            response = requests.post(url='http://med-3d.top:2334/sys/caseUpdate',
                                     headers={'_id': str(find['_id']), 'case_uid': case_uid})
            if response.text == 'true':
                case_db.find_one_and_update({'_id': find['_id']}, {'$set': case_set})
                vmi.askInfo('同步完成')
                return True
            else:
                vmi.askInfo('错误：' + response.text)
        except Exception as e:
            vmi.askInfo('错误：' + str(e))


def find_order_inspected():
    combo = QComboBox()
    for case in case_db.find():
        if case['customer_confirmed']:
            if case['order_released'] and case['order_inspected'] and not case['order_completed']:
                t = case['last_update_time'][:14]
                t = [t[:4], t[4:6], t[6:8], t[8:10], t[10:12], t[12:14]]
                t = '-'.join(t)
                combo.addItem(case['order_uid'] + ' ' + t, case['order_uid'])

    if not combo.count():
        vmi.askInfo('无在产生产单')
        return

    sizep = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
    dialog = QDialog()
    dialog.setSizePolicy(sizep)
    dialog.setWindowTitle('选择在产生产单')
    dialog.setLayout(QVBoxLayout())
    dialog.setMinimumWidth(200)
    dialog.layout().addWidget(combo)

    ok_cancel = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, dialog)
    ok_cancel.setSizePolicy(sizep)
    ok_cancel.accepted.connect(dialog.accept)
    ok_cancel.rejected.connect(dialog.reject)
    dialog.layout().addWidget(ok_cancel)

    if dialog.exec_() != QDialog.Accepted:
        return

    r = vmi.askButtons(['下载设计文件', '完成生产单'])
    if r == '下载设计文件':
        user = vmi.application.skyward.ask_user()
        if not user:
            return False

        role_quality_inspector_id = user['role_quality_inspector_id']
        if not role_quality_inspector_db.find_one({'_id': role_quality_inspector_id}):
            vmi.askInfo('错误：只有质量检验员可以下载设计文件')
            return False

        download_order_files(combo.currentData())
    elif r == '完成生产单':
        user = vmi.application.skyward.ask_user()
        if not user:
            return False

        role_quality_inspector_id = user['role_quality_inspector_id']
        if not role_quality_inspector_db.find_one({'_id': role_quality_inspector_id}):
            vmi.askInfo('错误：只有质量检验员可以完成生产单')
            return False

        case_set = {'role_quality_inspector_id': role_quality_inspector_id,
                    'order_released': True,
                    'order_inspected': True,
                    'order_completed': True}

        try:
            find = case_db.find_one({'order_uid': combo.currentData()})
            case_uid = vmi.convert_base(round(time.time() * 100))
            response = requests.post(url='http://med-3d.top:2334/sys/caseUpdate',
                                     headers={'_id': str(find['_id']), 'case_uid': case_uid})
            if response.text == 'true':
                case_db.find_one_and_update({'_id': find['_id']}, {'$set': case_set})
                vmi.askInfo('同步完成')
                return True
            else:
                vmi.askInfo('错误：' + response.text)
        except Exception as e:
            vmi.askInfo('错误：' + str(e))


if __name__ == '__main__':
    vmi.app.setApplicationName('Skyorder')
    vmi.app.setApplicationVersion('1.0')
    vmi.app.setQuitOnLastWindowClosed(False)

    client = pymongo.MongoClient('mongodb://root:medraw123@med-3d.top:27018/admin', 27017)
    database = client.skyward
    case_db = database.get_collection('case')
    order_db = database.get_collection('order')
    stl_fs = gridfs.GridFS(database, collection='stl')

    tray_icon_menu = QMenu()
    tray_icon_menu.addAction('待检生产单').triggered.connect(find_order_released)
    tray_icon_menu.addAction('在产生产单').triggered.connect(find_order_inspected)
    tray_icon_menu.addSeparator()
    tray_icon_menu.addAction('关于').triggered.connect(vmi.app_about)
    tray_icon_menu.addSeparator()
    tray_icon_menu.addAction('退出').triggered.connect(vmi.app.quit)

    tray_icon = QSystemTrayIcon()
    tray_icon.setIcon(QWidget().style().standardIcon(QStyle.SP_ComputerIcon))
    tray_icon.setToolTip('{} {}'.format(vmi.app.applicationName(), vmi.app.applicationVersion()))
    tray_icon.setContextMenu(tray_icon_menu)
    tray_icon.activated.connect(tray_icon_activated)
    tray_icon.show()

    vmi.app.exec_()
    vmi.appexit()



import pathlib
import shutil
import tempfile
from typing import Union, List, Optional, Dict, Any

import numpy as np
import vtk

import vmi


def pdOpenFile_STL(file=None) -> Optional[vtk.vtkPolyData]:
    """读取STL文件返回vtkPolyData，读取失败则返回None"""
    if file is None:
        file = vmi.askOpenFile('*.stl', '导入 (Import) STL')
        if file is None:
            return
    with tempfile.TemporaryDirectory() as p:
        p = pathlib.Path(p) / '.stl'
        shutil.copyfile(file, p)
        r = vtk.vtkSTLReader()
        r.SetFileName(str(p))
        r.Update()
        return r.GetOutput()


def pdSaveFile_STL(pd: vtk.vtkPolyData, file=None) -> None:
    """保存vtkPolyData到STL文件"""
    if file is None:
        file = vmi.askSaveFile('stl', '*.stl', '导出 (Export) STL')
        if file is None:
            return
    with tempfile.TemporaryDirectory() as p:
        p = pathlib.Path(p) / '.stl'
        w = vtk.vtkSTLWriter()
        w.SetInputData(pd)
        w.SetFileName(str(p))
        w.SetFileTypeToBinary()
        w.Update()
        shutil.copyfile(p, file)


def pdSaveFile_XML(pd: vtk.vtkPolyData, file=None) -> None:
    """保存vtkPolyData到vtp文件"""
    if file is None:
        file = vmi.askSaveFile('vtp', '*.vtp', '导出 (Export) XML')
        if file is None:
            return

    with tempfile.TemporaryDirectory() as p:
        p = pathlib.Path(p) / '.vtp'
        w = vtk.vtkXMLPolyDataWriter()
        w.SetFileName(str(p))
        w.SetInputData(pd)
        w.SetDataModeToBinary()
        w.SetCompressorTypeToZLib()
        w.Update()
        shutil.copyfile(p, file)


def pdOpenFile_XML(file=None) -> Optional[vtk.vtkPolyData]:
    """读取vtp文件返回vtkPolyData，读取失败则返回None"""
    if file is None:
        file = vmi.askOpenFile('*.vtp', '导入 (Import) XML')
        if file is None:
            return
    with tempfile.TemporaryDirectory() as p:
        p = pathlib.Path(p) / '.vtp'
        shutil.copyfile(file, p)
        r = vtk.vtkXMLPolyDataReader()
        r.SetFileName(str(p))
        r.Update()
        return r.GetOutput()


def pdBounds(pd: vtk.vtkPolyData):
    """返回边界[xmin, xmax, ymin, ymax, zmin, zmax]"""
    return np.array(pd.GetBounds())


def pdCenter(pd: vtk.vtkPolyData):
    """返回中心[x, y, z]"""
    b = pdBounds(pd)
    return 0.5 * np.array([b[0] + b[1], b[2] + b[3], b[4] + b[5]])


def pdSize(pd: vtk.vtkPolyData):
    """返回轴向包围盒尺寸"""
    b = pdBounds(pd)
    return np.linalg.norm(np.array([b[1] - b[0], b[3] - b[2], b[5] - b[4]]))


def pdPolyline(pts: List[Union[np.ndarray, List[float]]], closed=False) -> Optional[vtk.vtkPolyData]:
    """
    构造折线段

    :param pts: 节点
    :param closed: 闭合
    :return: vtkPolyData，构造失败则返回None
    """
    if len(pts) < 2:
        return

    points = vtk.vtkPoints()
    for pt in pts:
        points.InsertNextPoint([pt[0], pt[1], pt[2]])

    ids = vtk.vtkIdList()

    for i in range(points.GetNumberOfPoints() - 1):
        ids.InsertNextId(i)

    if closed:
        ids.InsertNextId(0)

    lines = vtk.vtkCellArray()
    lines.InsertNextCell(ids)

    pd = vtk.vtkPolyData()
    pd.SetPoints(points)
    pd.SetLines(lines)
    return pd


def pdPolyline_Regular(radius: float, center: Union[np.ndarray, List[float]], normal: Union[np.ndarray, List[float]],
                       resolution: float = 120) -> vtk.vtkPolyData:
    """
    构造无面正多边形

    :param radius: 节点半径
    :param center: 中心坐标
    :param normal: 所在平面的法向
    :param resolution: 边数
    :return: vtkPolyData，构造失败则返回None
    """
    polyline = vtk.vtkRegularPolygonSource()
    polyline.SetCenter(*list(center[:3]))
    polyline.SetNormal(*list(normal[:3]))
    polyline.SetRadius(radius)
    polyline.SetNumberOfSides(resolution)
    polyline.SetGeneratePolygon(0)
    polyline.Update()
    return polyline.GetOutput()


def pdPolygon_Regular(radius: float, center: Union[np.ndarray, List[float]], normal: Union[np.ndarray, List[float]],
                      resolution: float = 120) -> vtk.vtkPolyData:
    """
    构造有面正多边形

    :param radius: 节点半径
    :param center: 中心坐标
    :param normal: 所在平面的法向
    :param resolution: 边数
    :return: vtkPolyData，构造失败则返回None
    """
    polyline = vtk.vtkRegularPolygonSource()
    polyline.SetCenter(*list(center[:3]))
    polyline.SetNormal(*list(normal[:3]))
    polyline.SetRadius(radius)
    polyline.SetNumberOfSides(resolution)
    polyline.SetGeneratePolygon(1)
    polyline.Update()
    return polyline.GetOutput()


def pdSphere(radius: float, center: Union[np.ndarray, List[float]], resolution: float = 120) -> vtk.vtkPolyData:
    """
    构造球体

    :param radius: 半径
    :param center: 中心
    :param resolution: 边数
    :return: vtkPolyData，构造失败则返回None
    """
    source = vtk.vtkSphereSource()
    source.SetCenter(list(center[:3]))
    source.SetRadius(radius)
    source.SetPhiResolution(resolution)
    source.SetThetaResolution(resolution)
    source.Update()
    return source.GetOutput()


def pdExtract_Largest(pd: vtk.vtkPolyData) -> vtk.vtkPolyData:
    """提取最大的相连网格"""
    connectivity = vtk.vtkPolyDataConnectivityFilter()
    connectivity.SetInputData(pd)
    connectivity.SetExtractionModeToLargestRegion()
    connectivity.Update()
    return connectivity.GetOutput()


def pdExtract_Closest(pd: vtk.vtkPolyData, pt) -> vtk.vtkPolyData:
    """提取距离pt最近的相连网格"""
    connectivity = vtk.vtkPolyDataConnectivityFilter()
    connectivity.SetInputData(pd)
    connectivity.SetExtractionModeToClosestPointRegion()
    connectivity.SetClosestPoint([pt[i] for i in range(3)])
    connectivity.Update()
    return connectivity.GetOutput()


def pdSmooth_WindowedSinc(pd: vtk.vtkPolyData, iter_num=10) -> vtk.vtkPolyData:
    """光顺网格，窗函数法，迭代次数iter_num越大则光顺程度越高"""
    smooth = vtk.vtkWindowedSincPolyDataFilter()
    smooth.SetInputData(pd)
    smooth.SetNumberOfIterations(iter_num)
    smooth.SetPassBand(0.1)
    smooth.SetNonManifoldSmoothing(1)
    smooth.SetNormalizeCoordinates(1)
    smooth.SetFeatureAngle(60)
    smooth.SetBoundarySmoothing(0)
    smooth.SetFeatureEdgeSmoothing(0)
    smooth.Update()
    return smooth.GetOutput()


def pdSmooth_Laplacian(pd: vtk.vtkPolyData, iter_num=10) -> vtk.vtkPolyData:
    """光顺网格，拉普拉斯函数法，迭代次数iter_num越大则光顺程度越高"""
    smooth = vtk.vtkSmoothPolyDataFilter()
    smooth.SetInputData(pd)
    smooth.SetNumberOfIterations(iter_num)
    smooth.SetConvergence(0)
    smooth.SetRelaxationFactor(0.33)
    smooth.SetFeatureAngle(60)
    smooth.SetBoundarySmoothing(0)
    smooth.SetFeatureEdgeSmoothing(0)
    smooth.Update()
    return smooth.GetOutput()


def pdNormals(pd: vtk.vtkPolyData) -> vtk.vtkPolyData:
    """计算网格单元法向"""
    if pd.GetPointData().GetNormals() is None or pd.GetCellData().GetNormals() is None:
        normals = vtk.vtkPolyDataNormals()
        normals.SetInputData(pd)
        normals.SetComputePointNormals(1)
        normals.SetComputeCellNormals(1)
        normals.Update()
        return normals.GetOutput()
    return pd


def pdTriangle(pd: vtk.vtkPolyData, pass_verts=1, pass_lines=1) -> vtk.vtkPolyData:
    """
    三角网格化

    :param pd: 输入网格
    :param pass_verts: 是否保留点单元
    :param pass_lines: 是否保留线单元
    :return: vtkPolyData
    """
    triangle = vtk.vtkTriangleFilter()
    triangle.SetPassVerts(pass_verts)
    triangle.SetPassLines(pass_lines)
    triangle.SetInputData(pd)
    triangle.Update()
    return triangle.GetOutput()


def pdClip_Implicit(pd: vtk.vtkPolyData, f: vtk.vtkImplicitFunction) -> List[vtk.vtkPolyData]:
    """
    隐函数裁剪网格

    :param pd: 输入网格
    :param f: 隐函数
    :return: [vtkPolyData (隐函数大于0), vtkPolyData (隐函数小于0)]
    """
    clip = vtk.vtkClipPolyData()
    clip.SetClipFunction(f)
    clip.SetInputData(pdNormals(pd))
    clip.SetGenerateClippedOutput(1)
    clip.Update()
    return [clip.GetOutput(), clip.GetClippedOutput()]


def pdClip_Pd(pd: vtk.vtkPolyData, clip_pd: vtk.vtkPolyData) -> List[vtk.vtkPolyData]:
    """
    网格裁剪网格

    :param pd: 输入网格
    :param clip_pd: 裁剪网格
    :return: [vtkPolyData (裁剪网格外侧), vtkPolyData (裁剪网格内侧)]
    """
    clip_function = vtk.vtkImplicitPolyDataDistance()
    clip_function.SetInput(pdNormals(clip_pd))
    return pdClip_Implicit(pd, clip_function)


def pdCut_Implicit(pd: vtk.vtkPolyData, f: vtk.vtkImplicitFunction) -> vtk.vtkPolyData:
    """
    隐函数切割网格

    :param pd: 输入网格
    :param f: 隐函数
    :return: vtkPolyData (隐函数等于0)
    """
    cut = vtk.vtkCutter()
    cut.SetCutFunction(f)
    cut.SetInputData(pdNormals(pd))
    cut.SetGenerateCutScalars(0)
    cut.Update()

    pd = cut.GetOutput()
    pd.GetPointData().DeepCopy(vtk.vtkPolyData().GetPointData())
    pd.GetCellData().DeepCopy(vtk.vtkPolyData().GetCellData())
    return pd


def pdAppend(pds) -> vtk.vtkPolyData:
    """拼接网格"""
    append = vtk.vtkAppendPolyData()
    for pd in pds:
        append.AddInputData(pd)
    append.Update()
    return append.GetOutput()


def pdClean(pd: vtk.vtkPolyData, point_merging=1) -> vtk.vtkPolyData:
    """清理网格"""
    clean = vtk.vtkCleanPolyData()
    clean.SetInputData(pd)
    clean.SetPointMerging(point_merging)
    clean.Update()
    return clean.GetOutput()


def pdBoolean_Union(pd0: vtk.vtkPolyData, pd1: vtk.vtkPolyData) -> vtk.vtkPolyData:
    """布尔并网格"""
    boolean = vtk.vtkLoopBooleanPolyDataFilter()
    boolean.SetOperationToUnion()
    boolean.SetInputData(0, pdNormals(pd0))
    boolean.SetInputData(1, pdNormals(pd1))
    boolean.Update()
    return boolean.GetOutput()


def pdBoolean_Intersection(pd0: vtk.vtkPolyData, pd1: vtk.vtkPolyData) -> vtk.vtkPolyData:
    """布尔交网格"""
    boolean = vtk.vtkLoopBooleanPolyDataFilter()
    boolean.SetOperationToIntersection()
    boolean.SetInputData(0, pdNormals(pd0))
    boolean.SetInputData(1, pdNormals(pd1))
    boolean.Update()
    return boolean.GetOutput()


def pdBoolean_Difference(pd0: vtk.vtkPolyData, pd1: vtk.vtkPolyData) -> vtk.vtkPolyData:
    """布尔减网格"""
    boolean = vtk.vtkLoopBooleanPolyDataFilter()
    boolean.SetOperationToDifference()
    boolean.SetInputData(0, pdNormals(pd0))
    boolean.SetInputData(1, pdNormals(pd1))
    boolean.Update()
    return boolean.GetOutput()


def pdRayDistance_Pt(pt: Union[np.ndarray, List[float]], vt: Union[np.ndarray, List[float]], target: vtk.vtkPolyData,
                     base_kd=None, base_obb=None) -> Optional[float]:
    """
    点pt沿方向vt投射到网格base的距离

    :param pt: 输入点
    :param vt: 投射方向
    :param target: 目标网格
    :param base_kd: 目标网格的vtkKdTreePointLocator
    :param base_obb: 目标网格的vtkOBBTree
    :return: 投射距离，无交点则返回None
    """
    pt = np.array([pt[i] for i in range(3)])
    vt = np.array([vt[i] for i in range(3)])
    vt /= np.linalg.norm(vt)

    bnd = target.GetBounds()
    d1 = np.array([bnd[1] - bnd[0], bnd[3] - bnd[2], bnd[5] - bnd[4]])
    d1 = np.linalg.norm(d1)

    if base_kd is None:
        base_kd = vtk.vtkKdTreePointLocator()
        base_kd.SetDataSet(target)
        base_kd.BuildLocator()

    cid = base_kd.FindClosestPoint(pt)

    cpt = np.array(target.GetPoint(cid))
    d2 = np.linalg.norm(pt - cpt)

    pt_last = pt + (d1 + d2) * vt

    if base_obb is None:
        base_obb = vtk.vtkOBBTree()
        base_obb.SetDataSet(target)
        base_obb.BuildLocator()

    pts = vtk.vtkPoints()
    base_obb.IntersectWithLine(pt, pt_last, pts, None)

    if pts.GetNumberOfPoints() > 0:
        return np.linalg.norm(pt - np.array(pts.GetPoint(0)))
    else:
        return None


def pdRayCast_Pd(pd: vtk.vtkPolyData, vt: Union[np.ndarray, List[float]], target: vtk.vtkPolyData,
                 max_dist: float) -> vtk.vtkPolyData:
    """
    网格pd中包含的所有点沿方向vt投射到网格base，保持pd单元关系不变

    :param pd: 输入网格
    :param vt: 投射方向
    :param target: 目标网格
    :param max_dist: 最大投射距离限制
    :return:
    """
    vt = np.array([vt[i] for i in range(3)])
    vt /= np.linalg.norm(vt)

    base_kd = vtk.vtkKdTreePointLocator()
    base_kd.SetDataSet(target)
    base_kd.BuildLocator()

    base_obb = vtk.vtkOBBTree()
    base_obb.SetDataSet(target)
    base_obb.BuildLocator()

    out = vtk.vtkPolyData()
    out.DeepCopy(pd)

    for i in range(pd.GetNumberOfPoints()):
        pt = pd.GetPoint(i)
        d = pdRayDistance_Pt(pt, vt, target, base_kd, base_obb)

        if d is None or d > max_dist:
            d = max_dist

        pt = pt + d * vt
        out.GetPoints().SetPoint(i, pt)

    return out


def pdApproximate_Pd(pd: vtk.vtkPolyData, match_vt: Union[np.ndarray, List[float]],
                     target_poly: vtk.vtkPolyData, safe_radius: float = 1) -> vtk.vtkPolyData:
    """
    网格pd中包含的所有点沿方向vt一次近似到网格target，保持pd单元关系不变
    
    :param pd: 输入网格
    :param match_vt: 逼近方向
    :param target_poly: 目标网格
    :return: 
    """
    out = vtk.vtkPolyData()
    out.DeepCopy(pd)

    target_kd = vtk.vtkKdTreePointLocator()
    target_kd.SetDataSet(target_poly)
    target_kd.BuildLocator()

    target_obb = vtk.vtkOBBTree()
    target_obb.SetDataSet(target_poly)
    target_obb.BuildLocator()

    center = vmi.pdCenter(pd)

    for i in range(pd.GetNumberOfPoints()):
        pt = np.array(pd.GetPoint(i))
        vt_center = vmi.vtOnPlane(center - pt, match_vt)

        cs = vmi.CS4x4(axis0=vt_center, axis2=match_vt, origin=pt)
        cs.orthogonalize([1])
        cs.normalize()

        d = [pdRayDistance_Pt(p, match_vt, target_poly, target_kd, target_obb) for p in [
            cs.mpt([0, 0, 0]),
            cs.mpt([0, -safe_radius, 0]),
            cs.mpt([0, safe_radius, 0]),
            # cs.mpt([-safe_radius, 0, 0]),
            # cs.mpt([-safe_radius, -safe_radius, 0]),
            # cs.mpt([-safe_radius, safe_radius, 0]),
            cs.mpt([safe_radius, 0, 0]),
            cs.mpt([safe_radius, -safe_radius, 0]),
            cs.mpt([safe_radius, safe_radius, 0])]]
        d = [_ for _ in d if _ is not None]
        d = [_ for _ in d if _ < np.min(d) + 2 * safe_radius]
        d = np.mean(d) if len(d) else 0
        pt = pt + d * match_vt
        out.GetPoints().SetPoint(i, pt)
    return out


def pdOBB_Pts(pts: Union[List, vtk.vtkPolyData, vtk.vtkPoints]) -> Dict[str, Any]:
    """
    返回点列pts的OBB包围盒参数，点列分布的三个主轴方向和尺寸，以及分布中心

    :param pts: 输入点列
    :return: {'axis': 轴向量列表, 'size': 轴向尺寸, 'center': 中心, 'origin': 原点}
    """
    if isinstance(pts, vtk.vtkPolyData):
        pts = pts.GetPoints()
    elif not isinstance(pts, vtk.vtkPoints):
        points = vtk.vtkPoints()
        for pt in pts:
            points.InsertNextPoint([pt[0], pt[1], pt[2]])
        pts = points

    if pts.GetNumberOfPoints() > 0:
        args = {'origin': [0, 0, 0], 'axis': [[0, 0, 0], [0, 0, 0], [0, 0, 0]], 'size': [0, 0, 0]}
        vtk.vtkOBBTree().ComputeOBB(pts, args['origin'], args['axis'][0], args['axis'][1], args['axis'][2],
                                    args['size'])

        args['center'] = args['origin']
        for i in args:
            args[i] = np.array(args[i])

        for j in range(3):
            args['axis'][j] = args['axis'][j] / np.linalg.norm(args['axis'][j])
            args['center'] = [args['center'][i] + 0.5 * args['size'][j] * args['axis'][j][i] for i in range(3)]
        return args


def pdOffset_Normals(pd: vtk.vtkPolyData, d) -> vtk.vtkPolyData:
    """沿网格法向平移距离d"""
    pd = pdNormals(pd)
    pd_out = vtk.vtkPolyData()
    pd_out.DeepCopy(pd)

    normals = pd.GetPointData().GetNormals()
    for i in range(pd.GetNumberOfPoints()):
        n = [0, 0, 0]
        normals.GetTypedTuple(i, n)
        n = np.array(n)
        n /= np.linalg.norm(n)

        pt = np.array(pd.GetPoint(i)) + d * n
        pd_out.GetPoints().SetPoint(i, pt)
    return pd_out


def pdRuledSurface(pts0: Union[np.ndarray, List[float]], pts1: Union[np.ndarray, List[float]],
                   closed: bool = False) -> vtk.vtkPolyData:
    """由两个点列构造直纹面"""
    cells = vtk.vtkCellArray()
    points = vtk.vtkPoints()

    for i in range(len(pts0)):
        points.InsertNextPoint(pts0[i])
        points.InsertNextPoint(pts1[i])

        ids = vtk.vtkIdList()
        ids.InsertNextId(2 * i)
        ids.InsertNextId(2 * i + 1)
        ids.InsertNextId(2 * i + 2)
        cells.InsertNextCell(ids)

        ids = vtk.vtkIdList()
        ids.InsertNextId(2 * i + 3)
        ids.InsertNextId(2 * i + 2)
        ids.InsertNextId(2 * i + 1)
        cells.InsertNextCell(ids)

    pd = vtk.vtkPolyData()
    pd.SetPoints(points)
    pd.SetPolys(cells)
    return pd


def pdMatrix(pd: vtk.vtkPolyData, matrix: vtk.vtkMatrix4x4) -> vtk.vtkPolyData:
    """变换网格，变换矩阵matrix"""
    t = vtk.vtkTransform()
    t.SetMatrix(matrix)
    return pdTransform(pd, t)


def pdTransform(pd: vtk.vtkPolyData, t: vtk.vtkTransform):
    """变换网格，变换t"""
    tf = vtk.vtkTransformPolyDataFilter()
    tf.SetTransform(t)
    tf.SetInputData(pd)
    tf.Update()
    return tf.GetOutput()



import hashlib
import pickle
import threading
from typing import Dict, Any

import gridfs
import pydicom
import pymongo
import requests
import vmi
import vtk
from PySide2.QtCore import *
from PySide2.QtWidgets import *


def input_dicom():
    global series_list, dcmdir
    dcmdir = vmi.askDirectory('DICOM')

    if dcmdir is not None:
        series_list = vmi.sortSeries(dcmdir)
        input_series()


def input_series():
    global series_dict, series
    series = vmi.askSeries(series_list)
    if series:
        series_dict = series.toKeywordValue()
        input_image(series.read())


def input_image(image: vtk.vtkImageData):
    series_volume.setData(image)
    series_view.setCamera_FitAll()
    series_view.updateAtOnce()
    vmi.appupdate()


def update_series_opcity():
    series_volume.setOpacityScalar({series_opacity_range[0] - 1: 0,
                                    series_opacity_range[0]: 1,
                                    series_opacity_range[1]: 1,
                                    series_opacity_range[1] + 1: 0})
    series_volume.setColor({series_opacity_range[0]: [1, 1, 0.9],
                            series_opacity_range[1]: [1, 1, 0.1]})


def output_nifti():
    if series_dict:
        vmi.imSaveFile_NIFTI(series_volume.data())


def upload_dcm():
    if series:
        file_names = series.filenames()

        tag_list = []
        for i in file_names:
            d = {}
            with pydicom.dcmread(i) as ds:
                for e in ds.iterall():
                    if e.value.__class__ in [None, bool, int, float, str]:
                        d[e.keyword] = e.value
                    elif e.value.__class__ in [bytes]:
                        d[e.keyword] = '{} bytes'.format(len(e.value))
                    else:
                        d[e.keyword] = str(e.value)
            d = {kw: d[kw] for kw in sorted(d)}
            tag_list.append(d)

        if not file_db.exists({'StudyInstanceUID': tag_list[0]['StudyInstanceUID']}):
            for i, val in enumerate(file_names):
                single_file = open(val, 'rb').read()
                single_md5 = hashlib.md5(single_file).hexdigest()
                if not file_db.exists({'md5': single_md5}):
                    file_db.put(single_file, **tag_list[i])


def output_upload():
    if series:
        # 上传原始数据
        upload_dcm()

        # 查找用户账号
        user_list = []
        user_db: pymongo.collection.Collection = client.wby_testDB.DoctorCollection
        for x in user_db.find({'doctor_userName': {'$exists': True}}):
            user_list.append(x['doctor_userName'])

        # 标注信息
        label = [QLabel('销售姓名:'), QLabel('销售电话:'), QLabel('手术时间:')]
        text = []

        size_policy_min = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

        dialog = QDialog()
        dialog.setSizePolicy(size_policy_min)
        dialog.setWindowTitle('Upload')
        dialog.setLayout(QVBoxLayout())
        dialog.setMinimumWidth(200)

        combo_box = [QComboBox(), QComboBox()]
        combo_box[0].addItem("导板名称：THA手术导板")
        combo_box[0].addItem("导板名称：TKA手术导板")
        dialog.layout().addWidget(combo_box[0])

        for i in user_list:
            combo_box[1].addItem('用户姓名： ' + i)
        dialog.layout().addWidget(combo_box[1])

        for i in range(len(label)):
            text.append(QLineEdit())
            dialog.layout().addWidget(label[i])
            dialog.layout().addWidget(text[i])

        button = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, dialog)
        button.setSizePolicy(size_policy_min)
        button.accepted.connect(dialog.accept)
        button.rejected.connect(dialog.reject)
        dialog.layout().addWidget(button)

        if dialog.exec_() == QDialog.Accepted:
            if series_dict:
                # Web通讯
                data = {
                    "uuid": QUuid.createUuid().toString(),
                    "patientName": series_dict["PatientName"],
                    "patientGender": series_dict["PatientSex"],
                    "patientAge": series_dict["PatientAge"],
                    "guideType": 'THA',
                    "userName": '1'
                }

                response = requests.post("http://192.168.11.185:3333/sys/newUID", data=str(data))

                case_type = []
                if combo_box[0].currentIndex() == 0:
                    case_type = 'THA'
                elif combo_box[0].currentIndex() == 1:
                    case_type = 'TKA'

                case_userName = user_list[combo_box[1].currentIndex()]

                if response.text != "-1":
                    # 上传案例数据
                    case_uid = {'case_uid': response.text}
                    key_dict = case_db.find_one(case_uid)
                    val_dict = {**key_dict,
                                'case_studyInstanceUID': series_dict['StudyInstanceUID'],
                                'case_userName': case_userName,
                                'case_salesName': text[0].text(),
                                'case_salesPhoneNumber': text[1].text(),
                                'case_operationDate': text[2].text(),
                                'case_type': case_type}
                    case_db.find_one_and_replace(key_dict, val_dict)


def LeftButtonPressRelease(**kwargs):
    global series_opacity_range
    if kwargs['picked'] is series_opacity_range_box[0]:
        v = vmi.askInt(-10000, series_opacity_range[0], series_opacity_range[1])
        if v is not None:
            series_opacity_range[0] = v
        series_opacity_range_box[0].draw_text('min {:.0f}'.format(series_opacity_range[0]))
        update_series_opcity()
    elif kwargs['picked'] is series_opacity_range_box[1]:
        v = vmi.askInt(series_opacity_range[0], series_opacity_range[0], 10000)
        if v is not None:
            series_opacity_range[1] = v
        series_opacity_range_box[1].draw_text('min {:.0f}'.format(series_opacity_range[1]))
        update_series_opcity()


def NoButtonWheel(**kwargs):
    global series_opacity_range
    if kwargs['picked'] is series_opacity_range_box[0]:
        series_opacity_range[0] = min(max(series_opacity_range[0] + kwargs['delta'],
                                          -10000), series_opacity_range[1])
        series_opacity_range_box[0].draw_text('min {:.0f}'.format(series_opacity_range[0]))
        update_series_opcity()
    elif kwargs['picked'] is series_opacity_range_box[1]:
        series_opacity_range[1] = min(max(series_opacity_range[1] + kwargs['delta'],
                                          series_opacity_range[0]), 10000)
        series_opacity_range_box[1].draw_text('max {:.0f}'.format(series_opacity_range[1]))
        update_series_opcity()


def return_globals():
    return globals()


if __name__ == '__main__':
    client = pymongo.MongoClient('mongodb://root:medraw123@192.168.11.122:27017/admin', 27017)
    case_db: pymongo.collection.Collection = client.wby_testDB.case
    file_db = gridfs.GridFS(client.SkywardDB, collection='Dicom')

    dcmdir, series = [], []

    main = vmi.Main(return_globals)
    main.setAppName('dicom_uoloader')
    main.setAppVersion(vmi.version)
    main.excludeKeys += ['main']

    menu_input = main.menuBar().addMenu('输入')
    menu_input.addAction('DICOM').triggered.connect(input_dicom)
    menu_input.addAction('Series').triggered.connect(input_series)

    menu_output = main.menuBar().addMenu('输出')
    menu_output.addAction('NIFTI').triggered.connect(output_nifti)
    menu_output.addAction('上传').triggered.connect(output_upload)

    series_list, series_dict = [], {}
    series_view = vmi.View()
    series_view.setCamera_Coronal()
    series_volume = vmi.ImageVolume(series_view)

    series_opacity_range = [200, 3000]
    update_series_opcity()

    series_opacity_range_box = [vmi.TextBox(series_view, text='min {:.0f}'.format(series_opacity_range[0]),
                                            size=[0.1, 0.03], pos=[0, 0.03], anchor=[0, 0], pickable=True),
                                vmi.TextBox(series_view, text='max {:.0f}'.format(series_opacity_range[1]),
                                            size=[0.1, 0.03], pos=[0.1, 0.03], anchor=[0, 0], pickable=True)]

    for box in series_opacity_range_box:
        box.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
        box.mouse['NoButton']['Wheel'] = [NoButtonWheel]

    main.layout().addWidget(series_view, 0, 0, 1, 1)
    vmi.appexec(main)
    vmi.appexit()



import vmi
import vtk
import pydicom
import numpy as np
from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *
from numba import jit
# from skimage.transform import resize
import random


def _compute_size(p1, p2, p3):
    p1 = np.array(p1)
    p2 = np.array(p2)
    p3 = np.array(p3)

    v1 = np.linalg.norm(np.abs(p1 - p2))
    v2 = np.linalg.norm(np.abs(p1 - p3))
    v3 = np.linalg.norm(np.abs(p3 - p2))
    p = (v1 + v2 + v3) / 2
    s = pow(p * (p - v1) * (p - v2) * (p - v3), 1 / 2)
    print(s)
    return s


def compute_area_by_pts(pts):
    pts = np.array(pts)
    area = 0
    center = pts.sum(axis=0) / len(pts)
    print(pts)
    print(center)

    for i, j in enumerate(pts):
        if i < len(pts) - 1:
            part = _compute_size(pts[i], pts[i + 1], center)
            area += part
        elif i == len(pts) - 1:
            area += _compute_size(pts[i], pts[0], center)
    return round(area, 2)


def update_fill_ratio():
    fill_ratio = round((pow(pring_coil / 2 * (10 * 2.54), 2) * np.pi * filling_lenth) / tumor_pd_size, 4)
    voi_view_textboxes[-1].draw_text('填充率:{}%'.format(fill_ratio))


def compute_size(pd):
    triang = vtk.vtkTriangleFilter()
    triang.SetInputData(pd.data())
    triang.Update()
    triang_pd = vmi.PolyActor(voi_view, color=[0, 0, 0], visible=False, pickable=False)
    triang_pd.setData(triang.GetOutput())
    mass = vtk.vtkMassProperties()
    mass.SetInputData(triang_pd.data())
    mass.Update()
    return round(mass.GetVolume(), 2), round(mass.GetSurfaceArea(), 2)


def update_para_text_3_4():
    para_3_box.draw_text(
        '载流动脉参数:\n半径1:{}mm\n半径2:{}mm\n半径3:{}mm\n半径4:{}mm\n半径5:{}mm'.format(para_dict['radius_1'], para_dict['radius_2'],
                                                                           para_dict['radius_3'],
                                                                           para_dict['radius_4'],
                                                                           para_dict['radius_5']))
    para_4_box.draw_text('半径6:{}mm\n半径7:{}mm\n半径8:{}mm\n半径9:{}mm\n半径10:{}mm\n半径11:{}mm'.format(para_dict['radius_6'],
                                                                                               para_dict['radius_7'],
                                                                                               para_dict['radius_8'],
                                                                                               para_dict['radius_9'],
                                                                                               para_dict['radius_10'],
                                                                                               para_dict['radius_11']))


def update_para_text_1_2():
    para_1_box.draw_text(
        '动脉瘤参数:\n长:{}mm\n宽:{}mm\n高:{}mm\n体积:{}mm³'.format(para_dict['lenth'], para_dict['width'], para_dict['hight'],
                                                          para_dict['tumor_volum']))
    para_2_box.draw_text('\n截面积:{}.mm²\n瘤颈深度:{}.mm\n角一:{}°\n角二:{}°'.format(para_dict['tumor_area'], para_dict['depth'],
                                                                           para_dict['angle_1'], para_dict['angle_2']))


def compute_angle(angle_pts):
    print(angle_pts)
    angle_pts = np.array(angle_pts[:3])
    v1 = np.abs(angle_pts[1] - angle_pts[0])
    v2 = np.abs(angle_pts[2] - angle_pts[0])
    v3 = np.abs(angle_pts[2] - angle_pts[1])
    l1 = np.linalg.norm(v1)
    l2 = np.linalg.norm(v2)
    l3 = np.linalg.norm(v3)

    cos_angle = (pow(l3, 2) + pow(l1, 2) - pow(l2, 2)) / (2 * l1 * l3)
    angle = round((np.arccos(cos_angle) / np.pi) * 180, 2)
    print(v1, v2, v3, l1, l2, l3, cos_angle, angle)
    return angle


def build_brace():
    max_radius = np.array(radius_arr).max()
    if len(brace_pts) > 1:
        for i in range(len(brace_pts)):
            brace_props[i].setData(vmi.pdSphere(radius, brace_pts[i]))
            print(brace_pts)
        edge = vmi.ccEdge(vmi.ccBSpline_Kochanek(brace_pts))
        l = vmi.ccLength(edge)
        pts = vmi.ccUniformPts_Edge(edge, round(max(l / 0.01, 10)))  # 至少分割成两个点,每个点分十个
        brace_cline.setData(vmi.pdPolyline(pts))
        brace_cline_tube.SetInputData(brace_cline.data())

        brace_cline_tube.SetRadius(max_radius)
        brace_cline_tube.SetNumberOfSides(500)
        brace_cline_tube.Update()
        brace_cline.setData(brace_cline_tube.GetOutput())


def make_centers_on_top():
    for i in vessel_center_props:
        i.setAlwaysOnTop(True)
    tumor_center_prop.setAlwaysOnTop(True)


def build_pipe():
    if len(pipe_pts) > 1:
        for i in range(len(pipe_pts)):
            pipe_props[i].setData(vmi.pdSphere(radius, pipe_pts[i]))
            print(pipe_pts)
        edge = vmi.ccEdge(vmi.ccBSpline_Kochanek(pipe_pts))
        l = vmi.ccLength(edge)
        pts = vmi.ccUniformPts_Edge(edge, round(max(l / 0.01, 10)))  # 至少分割成两个点,每个点分十个
        pipe_cline.setData(vmi.pdPolyline(pts))
        pipe_cline_tube.SetInputData(pipe_cline.data())
        pipe_cline_tube.SetRadius(0.3)
        pipe_cline_tube.SetNumberOfSides(500)
        pipe_cline_tube.Update()
        pipe_cline.setData(pipe_cline_tube.GetOutput())


def update_tumor_curve(index):
    global tumor_center
    radius = 0.3
    if len(tumor_f_l) >= 1:
        for i in range(len(tumor_f_l)):
            tumor_f_l_props[i][0].setData(vmi.pdSphere(radius, tumor_f_l[i][0]))

            tumor_f_l_props[i][1].setData(vmi.pdSphere(radius, tumor_f_l[i][1]))
            print('tumor_f_l[i]:', tumor_f_l[i])
            tumor_line_props[i].setData(vmi.ccWire(vmi.ccSegments(tumor_f_l[i], False)))

    if len(tumor_f_l_props) == 3:
        _cp = np.array(tumor_f_l)
        tumor_center = _cp.sum(axis=0).sum(axis=0) / 6
        tumor_center_prop.setData(vmi.pdSphere(radius, tumor_center))
    if index[0][0] == 0:
        para_dict['lenth'] = round(
            np.linalg.norm(np.abs(np.array(tumor_f_l[index[0][0]][0]) - np.array(tumor_f_l[index[0][0]][1]))), 2)
    if index[0][0] == 1:
        para_dict['width'] = round(
            np.linalg.norm(np.abs(np.array(tumor_f_l[index[0][0]][0]) - np.array(tumor_f_l[index[0][0]][1]))), 2)
    if index[0][0] == 2:
        para_dict['hight'] = round(
            np.linalg.norm(np.abs(np.array(tumor_f_l[index[0][0]][0]) - np.array(tumor_f_l[index[0][0]][1]))), 2)
    update_para_text_1_2()
    voi_view.updateInTime()


def update_vessel_curve():
    global vessel_centers
    vessel_centers = []
    if len(vessel_f_l) >= 1:
        for i in range(len(vessel_f_l)):
            center = (np.array(vessel_f_l[i][0]) + np.array(vessel_f_l[i][1])) / 2
            vessel_f_l_props[i][0].setData(vmi.pdSphere(radius, vessel_f_l[i][0]))

            vessel_f_l_props[i][1].setData(vmi.pdSphere(radius, vessel_f_l[i][1]))
            print(vessel_f_l[i])
            vessel_line_props[i].setData(vmi.ccWire(vmi.ccSegments(vessel_f_l[i], False)))
            vessel_center_props[i].setData(vmi.pdSphere(radius, center))
            vessel_centers.append(center)
    voi_view.updateInTime()


def seg_ary_mask(input_ary, mask_ary, threshold):
    threshold_ary = input_ary >= threshold
    logic_mask = mask_ary == 255
    input_ary[np.logical_not(np.logical_and(logic_mask, threshold_ary))] = threshold - 16
    return input_ary


def rebulid_image(plcut_pts, image_1, image_2):
    # q1:旋转视角是否有效
    global voi_image
    cs = voi_view.cameraCS()
    d = vmi.imSize_Vt(voi_image, cs.axis(2)) * 2
    pts = [vmi.ptOnPlane(pt, vmi.imCenter(voi_image), cs.axis(2)) for pt in plcut_pts]
    pts = [pt - d * cs.axis(2) for pt in pts]
    sh = vmi.ccFace(vmi.ccWire(vmi.ccSegments(pts, True)))
    sh = vmi.ccPrism(sh, cs.axis(2), 2 * d)
    mask = vmi.imStencil_PolyData(vmi.ccPd_Sh(sh),
                                  vmi.imOrigin(voi_image),
                                  vmi.imSpacing(voi_image),
                                  vmi.imExtent(voi_image))
    mask_ary = vmi.imArray_VTK(mask)
    voi_ary = vmi.imArray_VTK(voi_image)
    tumor_ary = seg_ary_mask(voi_ary, mask_ary, window_level - 100)
    tumor_image = vmi.imVTK_Array(tumor_ary, vmi.imOrigin(voi_image), vmi.imSpacing(voi_image))

    fe3 = vtk.vtkFlyingEdges3D()
    fe3.SetInputData(tumor_image)
    fe3.SetValue(0, window_level + 50)
    fe3.SetComputeNormals(1)
    fe3.SetComputeGradients(0)
    fe3.SetComputeScalars(0)
    fe3.Update()

    pd = fe3.GetOutput()
    return pd


def ary_mask(input_ary, mask_ary, threshold, target):
    threshold_ary = input_ary >= threshold
    logic_mask = mask_ary == 255
    log_ary = np.logical_and(logic_mask, threshold_ary)
    input_ary[log_ary] = target


def update_line_width_cut(plcut_pts):
    if len(plcut_pts) > 1:
        plcut_prop.setData(vmi.ccWire(vmi.ccSegments(plcut_pts, True)))
    else:
        plcut_prop.setData(None)


def plcut_voi_image():
    global voi_image

    cs = voi_view.cameraCS()
    d = vmi.imSize_Vt(voi_image, cs.axis(2)) * 4
    pts = [vmi.ptOnPlane(pt, vmi.imCenter(voi_image), cs.axis(2)) for pt in plcut_pts]

    pts = [pt - d * cs.axis(2) for pt in pts]

    sh = vmi.ccFace(vmi.ccWire(vmi.ccSegments(pts, True)))

    sh = vmi.ccPrism(sh, cs.axis(2), 2 * d)

    mask = vmi.imStencil_PolyData(vmi.ccPd_Sh(sh),
                                  vmi.imOrigin(voi_image),
                                  vmi.imSpacing(voi_image),
                                  vmi.imExtent(voi_image))
    mask_ary = vmi.imArray_VTK(mask)
    voi_ary = vmi.imArray_VTK(voi_image)
    ary_mask(voi_ary, mask_ary, threshold, back_value)
    voi_image = vmi.imVTK_Array(voi_ary, vmi.imOrigin(voi_image), vmi.imSpacing(voi_image))
    voi_volume.setData(voi_image)


def draw_standard_line():
    '''勾画标定区域，用于确定阈值'''
    pass


@jit(nopython=True)
def ary_pad(input_ary, constant):
    for k in [0, input_ary.shape[0] - 1]:
        for j in range(input_ary.shape[1]):
            for i in range(input_ary.shape[2]):
                input_ary[k][j][i] = constant
    for k in range(input_ary.shape[0]):
        for j in [0, input_ary.shape[1] - 1]:
            for i in range(input_ary.shape[2]):
                input_ary[k][j][i] = constant
    for k in range(input_ary.shape[0]):
        for j in range(input_ary.shape[1]):
            for i in [0, input_ary.shape[2] - 1]:
                input_ary[k][j][i] = constant


def init_voi(voi_center, voi_size):
    global voi_image, voi_ary, tumor_image_data, vessel_image_data
    '''裁剪图像区域'''
    tumor_image_data = vtk.vtkImageData()
    vessel_image_data = vtk.vtkImageData()
    voi_origin = voi_center - 0.5 * voi_size
    voi_cs = vmi.CS4x4(origin=voi_origin)
    voi_image = vmi.imReslice(original_image, voi_cs, voi_size, -1024)
    voi_image.SetOrigin(*list(voi_cs.origin()))
    voi_ary = vmi.imArray_VTK(voi_image)
    ary_pad(voi_ary, -1024)
    voi_image = vmi.imVTK_Array(voi_ary, vmi.imOrigin(voi_image), vmi.imSpacing(voi_image))
    voi_volume.setData(voi_image)
    voi_volume.setOpacity_Threshold(window_level)
    voi_volume.setPickable(True)
    voi_view_textboxes[0].draw_text('阈值：{} HU'.format(window_level))
    tumor_image_data = tumor_image_data.DeepCopy(voi_image)
    vessel_image_data = vessel_image_data.DeepCopy(voi_image)


def on_init_voi():
    '''弹出选择尺寸窗口'''
    size = vmi.askInt(1, 30, 100, suffix='mm', title='请输入目标区域尺寸')
    if size is not None:
        original_target = original_view.pickPt_Cell()
        voi_center = np.array([vmi.imCenter(original_image)[0], original_target[1], original_target[2]])
        voi_size = np.array([size, size, size])
        init_voi(voi_center, voi_size)
        voi_view.setCamera_Coronal()
        voi_view.setCamera_FitAll()
        voi_view.setEnabled(True)


def read_dicom():
    dcmdir = vmi.askDirectory()  # 用户选择文件夹

    if dcmdir is not None:  # 判断用户选中了有效文件夹并点击了确认
        series_list = vmi.sortSeries(dcmdir)  # 将文件夹及其子目录包含的所有DICOM文件分类到各个系列

        if len(series_list) > 0:  # 判断该文件夹内包含有效的DICOM系列
            series = vmi.askSeries(series_list)  # 用户选择DICOM系列

            if series is not None:  # 判断用户选中了有效系列并点击了确认
                with pydicom.dcmread(series.filenames()[0]) as ds:
                    global patient_name
                    patient_name = ds.PatientName
                return series.read()  # 读取DICOM系列为图像数据


def LeftButtonPressMove(**kwargs):
    '''
    鼠标左键平移动作
    '''
    global tumor_center, section_pts, section_prop
    if kwargs['picked'] is original_slice:
        # 勾画标定区域
        if original_view_textboxes[1].text() == '正在勾画标定区域':
            pt = original_view.pickPt_FocalPlane()
            for path_pt in standard_pts:
                if (path_pt == pt).all():
                    return

            standard_pts.append(pt)  # 添加拾取路径点
            if len(section_pts) > 1:
                standard_prop.setData(vmi.ccWire(vmi.ccSegments(standard_pts, True)))
            else:
                standard_prop.setData(None)
        else:
            pass

    elif kwargs['picked'] is voi_view:
        if voi_view_textboxes[1].text() == '请勾画切割范围':
            pt = voi_view.pickPt_FocalPlane()

            for path_pt in plcut_pts:
                if (path_pt == pt).all():
                    return

            plcut_pts.append(pt)  # 添加拾取路径点
            update_line_width_cut(plcut_pts)
            return True
        elif voi_view_textboxes[2].text() == '请勾画动脉瘤':
            pt = voi_view.pickPt_FocalPlane()

            for path_pt in tumor_pts:
                if (path_pt == pt).all():
                    return

            tumor_pts.append(pt)  # 添加拾取路径点
            update_line_width_cut(tumor_pts)
            return True
        elif voi_view_textboxes[3].text() == '请勾画载流动脉':
            pt = voi_view.pickPt_FocalPlane()

            for path_pt in vessel_pts:
                if (path_pt == pt).all():
                    return

            vessel_pts.append(pt)  # 添加拾取路径点
            update_line_width_cut(vessel_pts)
            return True



        elif voi_view_textboxes[8].text() == '请勾画截面':
            pt = voi_view.pickPt_FocalPlane()

            for path_pt in section_pts:
                if (path_pt == pt).all():
                    return

            section_pts.append(pt)  # 添加拾取路径点
            if len(section_pts) > 1:
                section_prop.setData(vmi.ccWire(vmi.ccSegments(section_pts, True)))
            else:
                section_prop.setData(None)
            return True

        else:
            voi_view.mouseRotateFocal()
    elif kwargs['picked'] in np.array(tumor_f_l_props):
        if voi_view_textboxes[4].text() == '正在测量动脉瘤':
            _cp = np.array(tumor_f_l_props)
            index = np.argwhere(_cp == kwargs['picked'])
            # print(tumor_f_l)
            tumor_f_l[index[0][0]][index[0][1]] += voi_view.pickVt_FocalPlane() / 2
            # print(tumor_f_l)
            update_tumor_curve(index)
    elif kwargs['picked'] in np.array(vessel_f_l_props):
        if voi_view_textboxes[5].text() == '正在测量载流动脉':
            _cp = np.array(vessel_f_l_props)
            index = np.argwhere(_cp == kwargs['picked'])
            # print(vessel_f_l)
            vessel_f_l[index[0][0]][index[0][1]] += voi_view.pickVt_FocalPlane() / 2
            vessel_f_l_props[index[0][0]][index[0][1]].setData(
                vmi.pdSphere(radius, vessel_f_l[index[0][0]][index[0][1]]))
            # print(index,index[0][0])

            vessel_centers[index[0][0]] = np.array(vessel_f_l[index[0][0]][0] + vessel_f_l[index[0][0]][1]) / 2
            # print(vessel_centers[index[0][0]])
            vessel_line_props[index[0][0]].setData(
                vmi.ccWire(vmi.ccSegments([vessel_f_l[index[0][0]][0], vessel_f_l[index[0][0]][1]], False)))
            vessel_center_props[index[0][0]].setData(vmi.pdSphere(radius, vessel_centers[index[0][0]]))
            # update_vessel_curve()
    elif kwargs['picked'] in vessel_center_props:
        if voi_view_textboxes[5].text() == '正在测量载流动脉':
            i = vessel_center_props.index(kwargs['picked'])
            vessel_centers[i] += voi_view.pickVt_FocalPlane() / 2
            vessel_center_props[i].setData(vmi.pdSphere(radius, vessel_centers[i]))
    elif kwargs['picked'] is tumor_center_prop:
        if voi_view_textboxes[4].text() == '正在测量动脉瘤':
            tumor_center += voi_view.pickVt_FocalPlane() / 2
            tumor_center_prop.setData(vmi.pdSphere(radius, tumor_center))


def LeftButtonPress(**kwargs):
    pass


def LeftButtonPressMoveRelease(**kwargs):
    global tumor_pd, vessel_pd, tumor_pd_size, area, tumor_area
    if kwargs['picked'] is original_slice:
        # 勾画完目标区域提起鼠标更换文字
        if original_view_textboxes[1].text() == '正在勾画标定区域':
            original_view_textboxes[1].draw_text('标定区域')
        else:
            pass
    elif kwargs['picked'] is voi_view:
        if voi_view_textboxes[1].text() == '请勾画切割范围':
            voi_view_textboxes[1].draw_text('请耐心等待')
            plcut_voi_image()
            voi_view_textboxes[1].draw_text('线切割')

            plcut_pts.clear()
            update_line_width_cut(plcut_pts)

        elif voi_view_textboxes[2].text() == '请勾画动脉瘤':
            voi_view_textboxes[2].draw_text('请耐心等待')
            tumor_pd = vmi.PolyActor(voi_view, repres='wireframe', color=[0.8, 0.8, 1])  # 面绘制显示
            tumor_pd.setData(rebulid_image(tumor_pts, tumor_image_data, vessel_image_data))
            tumor_pd_size, tumor_area = compute_size(tumor_pd)
            para_dict['tumor_volum'] = tumor_pd_size
            update_para_text_1_2()
            voi_view_textboxes[2].draw_text('勾画动脉瘤')
            voi_view.setCamera_FitAll()
            tumor_pts.clear()
            update_line_width_cut(tumor_pts)
        elif voi_view_textboxes[3].text() == '请勾画载流动脉':
            voi_view_textboxes[3].draw_text('请耐心等待')
            vessel_pd = vmi.PolyActor(voi_view, repres='wireframe', color=[1, 0.8, 0.8])  # 面绘制显示
            vessel_pd.setData(rebulid_image(vessel_pts, vessel_image_data, vessel_image_data))
            voi_view_textboxes[3].draw_text('勾画载流动脉')

            vessel_pts.clear()
            update_line_width_cut(vessel_pts)
        elif voi_view_textboxes[8].text() == '请勾画截面':

            polygon = vtk.vtkPolygon()
            # polygon.GetPointIDs.Se
            # para_dict['tumor_area'] = np.round(tumor_area/6,2)
            para_dict['tumor_area'] = compute_area_by_pts(section_pts)
            section_prop.setData(None)
            update_para_text_1_2()
            voi_view_textboxes[8].draw_text('截面测量')


def LeftButtonPressRelease(**kwargs):
    # print(kwargs['picked'])
    global pipe_props, pipe_pts, brace_pts, brace_props, \
        angle_1_pts, angle_1_props, angle_2_pts, angle_2_props, \
        angle_1_line_1, angle_1_line_2, angle_2_line_1, angle_2_line_2, \
        line, line_pts, line_props, line_measure, radius_arr, section_pts, section_prop
    if kwargs['double'] is True:
        if kwargs['picked'] is voi_view:

            if voi_view_textboxes[4].text() == '正在测量动脉瘤':
                if len(tumor_f_l_props) < 3:
                    pt = voi_view.pickPt_FocalPlane()
                    look = voi_view.cameraVt_Look()
                    pt_f = pt - 2000 * look
                    pt_b = pt + 2000 * look
                    obb = vtk.vtkOBBTree()
                    obb.SetDataSet(tumor_pd.data())
                    obb.BuildLocator()

                    intersect_points = vtk.vtkPoints()
                    cell_id_list = vtk.vtkIdList()

                    interseced = obb.IntersectWithLine(pt_f, pt_b, intersect_points, cell_id_list)
                    first_pt, end_pt = [0, 0, 0], [0, 0, 0]
                    if interseced:
                        intersect_points.GetPoint(0, first_pt)
                        num = intersect_points.GetNumberOfPoints()
                        if num >= 2:
                            intersect_points.GetPoint(num - 1, end_pt)
                    if first_pt != [0, 0, 0] and end_pt != [0, 0, 0]:
                        tumor_prop_f = vmi.PolyActor(voi_view, color=[0.4, 0.4, 1], pickable=True)
                        # tumor_prop_f.setAlwaysOnTop(True)
                        tumor_prop_f.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                        tumor_prop_f.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
                        tumor_prop_b = vmi.PolyActor(voi_view, color=[0.4, 0.4, 1], pickable=True)
                        # tumor_prop_b.setAlwaysOnTop(True)
                        tumor_prop_b.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                        tumor_prop_b.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
                        tumor_line_prop = vmi.PolyActor(voi_view, color=[0.4, 0.4, 1], line_width=3, pickable=False)
                        tumor_f_l.append([first_pt, end_pt])
                        props_f_l = [tumor_prop_f, tumor_prop_b]
                        tumor_f_l_props.append(props_f_l)
                        tumor_line_props.append(tumor_line_prop)
                        update_tumor_curve([[5, 5]])
                        if len(tumor_f_l) == 1:
                            para_dict['lenth'] = round(
                                np.linalg.norm(np.abs(np.array(tumor_f_l[-1][0]) - np.array(tumor_f_l[-1][1]))), 2)
                        elif len(tumor_f_l) == 2:
                            para_dict['width'] = round(
                                np.linalg.norm(np.abs(np.array(tumor_f_l[-1][0]) - np.array(tumor_f_l[-1][1]))), 2)
                        elif len(tumor_f_l) == 3:
                            para_dict['hight'] = round(
                                np.linalg.norm(np.abs(np.array(tumor_f_l[-1][0]) - np.array(tumor_f_l[-1][1]))), 2)
                        update_para_text_1_2()
                    return True
            elif voi_view_textboxes[5].text() == '正在测量载流动脉':
                pt = voi_view.pickPt_FocalPlane()
                look = voi_view.cameraVt_Look()
                pt_f = pt - 2000 * look
                pt_b = pt + 2000 * look
                obb = vtk.vtkOBBTree()
                obb.SetDataSet(vessel_pd.data())
                obb.BuildLocator()

                intersect_points = vtk.vtkPoints()
                cell_id_list = vtk.vtkIdList()

                interseced = obb.IntersectWithLine(pt_f, pt_b, intersect_points, cell_id_list)
                first_pt, end_pt = [0, 0, 0], [0, 0, 0]
                if interseced:
                    intersect_points.GetPoint(0, first_pt)
                    num = intersect_points.GetNumberOfPoints()
                    if num >= 2:
                        intersect_points.GetPoint(num - 1, end_pt)
                if first_pt != [0, 0, 0] and end_pt != [0, 0, 0]:
                    vessel_prop_f = vmi.PolyActor(voi_view, color=[1, 0.8, 0.8], pickable=True)
                    # vessel_prop_f.setAlwaysOnTop(True)
                    vessel_prop_f.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                    vessel_prop_f.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]

                    vessel_prop_b = vmi.PolyActor(voi_view, color=[1, 0.8, 0.8], pickable=True)
                    # vessel_prop_b.setAlwaysOnTop(True)
                    vessel_prop_b.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                    vessel_prop_b.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]

                    vessel_line_prop = vmi.PolyActor(voi_view, color=[1, 0.8, 0.8], line_width=3, pickable=False)
                    vessel_line_prop.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                    vessel_line_prop.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]

                    vessel_center_prop = vmi.PolyActor(voi_view, color=[1, 0.2, 0.2], pickable=True)
                    # vessel_center_prop.setAlwaysOnTop(True)
                    vessel_center_prop.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
                    vessel_center_prop.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
                    vessel_center_prop.mouse['NoButton']['Wheel'] = NoButtonWheel

                    vessel_f_l.append([first_pt, end_pt])
                    props_f_l = []  # 一对表面点
                    props_f_l.append(vessel_prop_f)
                    props_f_l.append(vessel_prop_b)
                    vessel_f_l_props.append(props_f_l)  # 表面props列表[[p1,p2]]
                    vessel_line_props.append(vessel_line_prop)  # 表面两点连线prop列表
                    vessel_center_props.append(vessel_center_prop)
                    update_vessel_curve()
                    update_para_text_3_4()
                return True


    # 勾画标定区域
    elif kwargs['picked'] is original_view_textboxes[1]:
        standard_prop.setData(None)
        standard_pts.clear()
        if original_view_textboxes[1].text() == '标定区域':
            original_view_textboxes[1].draw_text('正在勾画标定区域')
            return True
        elif original_view_textboxes[1].text() == '正在勾画标定区域':
            original_view_textboxes[1].draw_text('标定区域')
            return True
    # 线切割按钮
    elif kwargs['picked'] is voi_view_textboxes[1]:
        if voi_view_textboxes[1].text() == '线切割':
            voi_view_textboxes[1].draw_text('请勾画切割范围')
            voi_view_textboxes[2].draw_text('勾画动脉瘤')
            voi_view_textboxes[3].draw_text('勾画载流动脉')
            return True
        elif voi_view_textboxes[1].text() == '请勾画切割范围':
            voi_view_textboxes[1].draw_text('线切割')
            return True
    # 勾画动脉瘤
    elif kwargs['picked'] is voi_view_textboxes[2]:
        if voi_view_textboxes[2].text() == '勾画动脉瘤':
            voi_view_textboxes[2].draw_text('请勾画动脉瘤')
            voi_view_textboxes[1].draw_text('线切割')
            voi_view_textboxes[3].draw_text('勾画载流动脉')
            return True
        elif voi_view_textboxes[2].text() == '请勾画动脉瘤':
            voi_view_textboxes[2].draw_text('勾画动脉瘤')
            return True
    # 勾画载流动脉
    elif kwargs['picked'] is voi_view_textboxes[3]:
        if voi_view_textboxes[3].text() == '勾画载流动脉':
            voi_view_textboxes[3].draw_text('请勾画载流动脉')
            voi_view_textboxes[1].draw_text('线切割')
            voi_view_textboxes[2].draw_text('勾画动脉瘤')
            return True
        elif voi_view_textboxes[3].text() == '请勾画载流动脉':
            voi_view_textboxes[3].draw_text('勾画载流动脉')
            return True
    # 测量动脉瘤
    elif kwargs['picked'] is voi_view_textboxes[4]:
        if voi_view_textboxes[4].text() == '测量动脉瘤':
            voi_view_textboxes[4].draw_text('正在测量动脉瘤')
            voi_view_textboxes[5].draw_text('测量载流动脉')
        elif voi_view_textboxes[4].text() == '正在测量动脉瘤':
            voi_view_textboxes[4].draw_text('测量动脉瘤')

        return True
    # 测量载流动脉
    elif kwargs['picked'] is voi_view_textboxes[5]:
        if voi_view_textboxes[5].text() == '测量载流动脉':
            voi_view_textboxes[5].draw_text('正在测量载流动脉')
            voi_view_textboxes[4].draw_text('测量动脉瘤')
            return True
        elif voi_view_textboxes[5].text() == '正在测量载流动脉':
            for i, f_l in enumerate(vessel_f_l):
                l = round(np.linalg.norm(abs(np.array(f_l[0]) - np.array(f_l[1]))) / 2, 2)
                para_dict['radius_' + str(i + 1)] = l
                radius_arr.append(l)

            print(para_dict)
            update_para_text_3_4()
            voi_view_textboxes[5].draw_text('测量载流动脉')

        return True
    # 选择导管路径
    elif kwargs['picked'] is voi_view_textboxes[6]:
        if voi_view_textboxes[6].text() == '路径规划':
            pipe_pts, pipe_props, pipe_wires, pipe_wires_props = [], [], [], []
            voi_view_textboxes[6].draw_text('选择导管路径')
            voi_view_textboxes[7].draw_text('支撑设计')
            voi_view_textboxes[10].draw_text('选择角一')
            voi_view_textboxes[11].draw_text('选择角二')
            make_centers_on_top()
            return True
        elif voi_view_textboxes[6].text() == '选择导管路径':
            print(22222)
            voi_view_textboxes[6].draw_text('正在生成导管路径...')
            # 生成导管模型

            build_pipe()
            voi_view_textboxes[6].draw_text('路径规划')
            for i in pipe_props:
                i.setVisible(False)

            return True
    # 选择支撑路径
    elif kwargs['picked'] is voi_view_textboxes[7]:
        if voi_view_textboxes[7].text() == '支撑设计':
            brace_pts, brace_props = [], []
            voi_view_textboxes[7].draw_text('选择支撑路径')
            voi_view_textboxes[6].draw_text('路径规划')
            voi_view_textboxes[10].draw_text('选择角一')
            voi_view_textboxes[11].draw_text('选择角二')
            make_centers_on_top()
            return True
        elif voi_view_textboxes[7].text() == '选择支撑路径':
            print(22222)
            voi_view_textboxes[7].draw_text('正在生成支撑...')
            # 生成支撑模型
            build_brace()
            voi_view_textboxes[7].draw_text('支撑设计')
            for i in brace_props:
                i.setData(None)

        return True
    elif kwargs['picked'] is voi_view_textboxes[8]:
        if voi_view_textboxes[8].text() == '截面测量':
            section_pts = []
            section_prop = vmi.PolyActor(voi_view, color=[1, 0.4, 0.4], line_width=3)

            voi_view_textboxes[10].draw_text('选择角一')
            voi_view_textboxes[6].draw_text('路径规划')
            voi_view_textboxes[7].draw_text('支撑设计')
            voi_view_textboxes[8].draw_text('请勾画截面')
            voi_view_textboxes[9].draw_text('长度测量')
            voi_view_textboxes[11].draw_text('选择角二')
        elif voi_view_textboxes[8].text() == '请勾画截面':
            voi_view_textboxes[8].draw_text('截面测量')
        return True
    elif kwargs['picked'] is voi_view_textboxes[9]:
        if voi_view_textboxes[9].text() == '长度测量':
            line, line_pts, line_props = 0, [], []
            line_measure = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)
            make_centers_on_top()
            voi_view_textboxes[10].draw_text('选择角一')
            voi_view_textboxes[6].draw_text('路径规划')
            voi_view_textboxes[7].draw_text('支撑设计')
            voi_view_textboxes[8].draw_text('截面测量')
            voi_view_textboxes[9].draw_text('任选两点测量长度')
            voi_view_textboxes[11].draw_text('选择角二')
        elif voi_view_textboxes[9].text() == '任选两点测量长度':
            voi_view_textboxes[9].draw_text('长度测量')
            for i in line_props:
                i.setVisible(False)
                line_measure.setVisible(False)
        return True
    elif kwargs['picked'] is voi_view_textboxes[10]:
        if voi_view_textboxes[10].text() == '选择角一':
            angle_1_pts, angle_1_props = [], []
            voi_view_textboxes[10].draw_text('选择角一顶点')
            voi_view_textboxes[6].draw_text('路径规划')
            voi_view_textboxes[7].draw_text('支撑设计')
            voi_view_textboxes[8].draw_text('截面测量')
            voi_view_textboxes[9].draw_text('长度测量')
            voi_view_textboxes[11].draw_text('选择角二')
            make_centers_on_top()
        elif voi_view_textboxes[10].text() == '选择角一顶点':
            voi_view_textboxes[10].draw_text('正在计算角一...')
            # 生成支撑模型
            if len(angle_1_pts) == 3:
                angle_1 = compute_angle(angle_1_pts)
                para_dict['angle_1'] = angle_1
                update_para_text_1_2()
            voi_view_textboxes[10].draw_text('选择角一')
            for i in angle_1_props:
                i.setVisible(False)
            angle_1_line_1.setVisible(False)
            angle_1_line_2.setVisible(False)
        return True
    elif kwargs['picked'] is voi_view_textboxes[11]:
        if voi_view_textboxes[11].text() == '选择角二':
            angle_2_pts, angle_2_props = [], []
            voi_view_textboxes[11].draw_text('选择角二顶点')
            voi_view_textboxes[6].draw_text('路径规划')
            voi_view_textboxes[7].draw_text('支撑设计')
            voi_view_textboxes[8].draw_text('截面测量')
            voi_view_textboxes[9].draw_text('长度测量')
            voi_view_textboxes[10].draw_text('选择角一')
            make_centers_on_top()
            return True
        if voi_view_textboxes[11].text() == '选择角二顶点':
            voi_view_textboxes[11].draw_text('正在计算角二...')
            # 生成支撑模型
            if len(angle_2_pts) == 3:
                angle_2 = compute_angle(angle_2_pts)
                para_dict['angle_2'] = angle_2
                update_para_text_1_2()
            voi_view_textboxes[11].draw_text('选择角二')
            for i in angle_2_props:
                i.setVisible(False)
            angle_2_line_1.setVisible(False)
            angle_2_line_2.setVisible(False)
            return True


    # 隐藏体数据
    elif kwargs['picked'] is voi_view_textboxes[12]:
        if voi_view_textboxes[12].text() == '隐藏整体数据':
            voi_volume.setVisible(False)
            voi_view_textboxes[12].draw_text('显示整体数据')
            return True
        else:
            voi_volume.setVisible(True)
            voi_view_textboxes[12].draw_text('隐藏整体数据')
            return True
    # 隐藏动脉瘤
    elif kwargs['picked'] is voi_view_textboxes[13]:
        if voi_view_textboxes[13].text() == '隐藏动脉瘤':
            tumor_pd.setVisible(False)
            voi_view_textboxes[13].draw_text('显示动脉瘤')
            return True

        else:
            tumor_pd.setVisible(True)
            voi_view_textboxes[13].draw_text('隐藏动脉瘤')

            return True

    # 隐藏载流动脉
    elif kwargs['picked'] is voi_view_textboxes[14]:
        if voi_view_textboxes[14].text() == '隐藏载流动脉':
            vessel_pd.setVisible(False)
            voi_view_textboxes[14].draw_text('显示载流动脉')
            return True

        else:
            vessel_pd.setVisible(True)
            voi_view_textboxes[14].draw_text('隐藏载流动脉')
            return True

    # 隐藏表面点
    elif kwargs['picked'] is voi_view_textboxes[15]:
        if voi_view_textboxes[15].text() == '隐藏表面点':
            for tumor_prop_f_l in tumor_f_l_props:
                for j in tumor_prop_f_l:
                    j.setVisible(False)
            for tumor_line in tumor_line_props:
                tumor_line.setVisible(False)
            for vessel_prop_f_l in vessel_f_l_props:
                for j in vessel_prop_f_l:
                    j.setVisible(False)
            for vessel_line in vessel_line_props:
                vessel_line.setVisible(False)
            voi_view_textboxes[15].draw_text('显示表面点')


        elif voi_view_textboxes[15].text() == '显示表面点':
            for tumor_prop_f_l in tumor_f_l_props:
                for j in tumor_prop_f_l:
                    j.setVisible(True)
            for tumor_line in tumor_line_props:
                tumor_line.setVisible(True)
            for vessel_prop_f_l in vessel_f_l_props:
                for j in vessel_prop_f_l:
                    j.setVisible(True)
            for vessel_line in vessel_line_props:
                vessel_line.setVisible(True)
            voi_view_textboxes[15].draw_text('隐藏表面点')
        return True
    elif kwargs['picked'] is voi_view_textboxes[16]:
        if voi_view_textboxes[16].text() == '隐藏导管路径':
            pipe_cline.setVisible(False)
            voi_view_textboxes[16].draw_text('显示导管路径')
        elif voi_view_textboxes[16].text() == '显示导管路径':
            pipe_cline.setVisible(True)
            voi_view_textboxes[16].draw_text('隐藏导管路径')
        return True
    elif kwargs['picked'] is voi_view_textboxes[17]:
        if voi_view_textboxes[17].text() == '隐藏支撑':
            brace_cline.setVisible(False)
            voi_view_textboxes[17].draw_text('显示支撑')
        elif voi_view_textboxes[17].text() == '显示支撑':
            brace_cline.setVisible(True)
            voi_view_textboxes[17].draw_text('隐藏支撑')
        return True
    # 选择路径，添加绿色的点实时刷新，点生成路径不显示
    if voi_view_textboxes[6].text() == '选择导管路径':
        if kwargs['picked'] in vessel_center_props:
            i = vessel_center_props.index(kwargs['picked'])
            print(vessel_centers[i], vessel_centers)
            if np.array(vessel_centers[i]) not in np.array(pipe_pts):
                pipe_pts.append(vessel_centers[i])
                pipe_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                pipe_props.append(pipe_prop)
                pipe_props[-1].setData(vmi.pdSphere(radius, vessel_centers[i]))
            return True
        elif kwargs['picked'] is tumor_center_prop:
            if np.array(tumor_center) not in np.array(pipe_pts):
                pipe_pts.append(tumor_center)
                pipe_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                pipe_props.append(pipe_prop)
                pipe_props[-1].setData(vmi.pdSphere(radius, tumor_center))
            return True
    if voi_view_textboxes[7].text() == '选择支撑路径':
        if kwargs['picked'] in vessel_center_props:
            i = vessel_center_props.index(kwargs['picked'])
            # print(vessel_centers[i], vessel_centers)
            if np.array(vessel_centers[i]) not in np.array(brace_pts):
                brace_pts.append(vessel_centers[i])
                brace_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                brace_props.append(brace_prop)
                brace_props[-1].setData(vmi.pdSphere(radius, vessel_centers[i]))
        elif kwargs['picked'] is tumor_center_prop:
            if np.array(tumor_center) not in np.array(brace_pts):
                brace_pts.append(tumor_center)
                brace_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                brace_props.append(brace_prop)
                brace_props[-1].setData(vmi.pdSphere(radius, tumor_center))
        return True
    if voi_view_textboxes[9].text() == '任选两点测量长度':
        print('line_pts', line_pts)
        if len(line_pts) < 2:
            print(kwargs['picked'])
            if kwargs['picked'] in vessel_center_props:
                print(11111)
                i = vessel_center_props.index(kwargs['picked'])
                # print(vessel_centers[i], vessel_centers)
                if np.array(vessel_centers[i]) not in np.array(line_pts):
                    line_pts.append(vessel_centers[i])
                    line_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    line_props.append(line_prop)
                    line_props[-1].setData(vmi.pdSphere(radius, vessel_centers[i]))
            elif kwargs['picked'] is tumor_center_prop:
                if np.array(tumor_center) not in np.array(line_pts):
                    line_pts.append(tumor_center)
                    line_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    line_props.append(line_prop)
                    line_props[-1].setData(vmi.pdSphere(radius, tumor_center))
        if len(line_pts) == 2:
            print(line_pts)
            line_measure.setData(vmi.ccWire(vmi.ccSegments(np.array(line_pts), False)))
            line = round(np.linalg.norm(np.abs(np.array(line_pts[0]) - np.array(line_pts[1]))), 2)
            para_dict['depth'] = line
            update_para_text_1_2()
        return True

    if voi_view_textboxes[10].text() == '选择角一顶点':
        print('angle_1_pts', angle_1_pts)

        if len(angle_1_pts) < 3:

            if kwargs['picked'] in vessel_center_props:
                i = vessel_center_props.index(kwargs['picked'])
                if np.array(vessel_centers[i]) not in np.array(angle_1_pts):
                    angle_1_pts.append(vessel_centers[i])
                    angle_1_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    angle_1_props.append(angle_1_prop)
                    angle_1_props[-1].setData(vmi.pdSphere(radius, vessel_centers[i]))
                    print(111)
            elif kwargs['picked'] is tumor_center_prop:
                if np.array(tumor_center) not in np.array(angle_1_pts):
                    angle_1_pts.append(tumor_center)
                    angle_1_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    angle_1_props.append(angle_1_prop)
                    angle_1_props[-1].setData(vmi.pdSphere(radius, tumor_center))
                    print(222)
        if len(angle_1_pts) == 2:
            # print('angle_1_pts:', angle_1_pts)
            angle_1_line_1 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3, always_on_top=True)
            angle_1_line_1.setData(vmi.ccWire(vmi.ccSegments(angle_1_pts, False)))
            print(333, angle_1_line_1)
        if len(angle_1_pts) == 3:
            # print('angle_1_pts_2:', angle_1_pts)

            angle_1_line_2 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3, always_on_top=True)
            angle_1_line_2.setData(vmi.ccWire(vmi.ccSegments(angle_1_pts[1:3], False)))

        return True
    if voi_view_textboxes[11].text() == '选择角二顶点':
        print('angle_2_pts', angle_2_pts)

        if len(angle_2_pts) < 3:

            if kwargs['picked'] in vessel_center_props:
                i = vessel_center_props.index(kwargs['picked'])
                if np.array(vessel_centers[i]) not in np.array(angle_2_pts):
                    angle_2_pts.append(vessel_centers[i])
                    angle_2_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    angle_2_props.append(angle_2_prop)
                    angle_2_props[-1].setData(vmi.pdSphere(radius, vessel_centers[i]))
                    print(111)
            elif kwargs['picked'] is tumor_center_prop:
                if np.array(tumor_center) not in np.array(angle_2_pts):
                    angle_2_pts.append(tumor_center)
                    angle_2_prop = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], pickable=True, always_on_top=True)
                    angle_2_props.append(angle_2_prop)
                    angle_2_props[-1].setData(vmi.pdSphere(radius, tumor_center))
                    print(222)
        if len(angle_2_pts) == 2:
            # print('angle_2_pts:', angle_2_pts)
            angle_2_line_1 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3, always_on_top=True)
            angle_2_line_1.setData(vmi.ccWire(vmi.ccSegments(angle_2_pts, False)))
            print(333, angle_2_line_1)
        if len(angle_2_pts) == 3:
            # print('angle_2_pts_2:', angle_2_pts)

            angle_2_line_2 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3, always_on_top=True)
            angle_2_line_2.setData(vmi.ccWire(vmi.ccSegments(angle_2_pts[1:3], False)))
            print(444, angle_2_line_2)

        return True


def NoButtonWheel(**kwargs):
    global window_level, window_width, filling_lenth, pring_coil
    # 修改窗位
    if kwargs['picked'] is original_view_textboxes[2]:
        window_level = min(max(window_level + 25 * kwargs['delta'], -1000), 3000)
        original_slice.setColorWindowLevel(window_level)
        original_view_textboxes[2].draw_text('窗位：{}'.format(window_level))
        return True
    # 修改窗宽
    elif kwargs['picked'] is original_view_textboxes[3]:
        window_width = min(max(window_width + 25 * kwargs['delta'], 0), 3000)
        original_slice.setColorWindowWidth(window_width)
        original_view_textboxes[3].draw_text('窗宽：{}'.format(window_width))
        return True
    elif kwargs['picked'] is voi_view_textboxes[-3]:
        pring_coil = round(min(max(pring_coil + 0.0001 * kwargs['delta'], 0), 3000), 5)
        voi_view_textboxes[-3].draw_text('弹簧圈直径:{}in'.format(pring_coil))
        update_fill_ratio()
    elif kwargs['picked'] is voi_view_textboxes[-2]:
        filling_lenth = min(max(filling_lenth + 1 * kwargs['delta'], 0), 3000)
        voi_view_textboxes[-2].draw_text('填充长度:{}mm'.format(filling_lenth))
        update_fill_ratio()


class Main(QScrollArea):
    def __init__(self):
        QScrollArea.__init__(self)
        brush = QBrush(QColor(255, 255, 255, 255))
        palette = QPalette()
        palette.setBrush(QPalette.Active, QPalette.Base, brush)
        palette.setBrush(QPalette.Active, QPalette.Window, brush)
        palette.setBrush(QPalette.Inactive, QPalette.Base, brush)
        palette.setBrush(QPalette.Inactive, QPalette.Window, brush)
        palette.setBrush(QPalette.Disabled, QPalette.Base, brush)
        palette.setBrush(QPalette.Disabled, QPalette.Window, brush)
        self.setPalette(palette)
        self.setWidgetResizable(True)

    def closeEvent(self, ev: QCloseEvent):
        ev.accept() if vmi.askYesNo('退出程序？') else ev.ignore()

    # vtk.vtkKdTreePointLocator


if __name__ == '__main__':
    patient_name = str()
    original_image = read_dicom()  # 读取DICOM路径
    if original_image is None:
        vmi.appexit()
    radius = 0.3
    radius_arr = []
    pring_coil = 0.0108
    filling_lenth = 0
    fill_ratio = 0

    color_dict = {'标定线': [0.2, 0.2, 1],
                  '线切割': [0.4, 0.4, 0.4],
                  '动脉瘤': [1, 0.4, 0.4],

                  '载流动脉': [0.4, 1, 0.4],

                  '路径点': [1, 0.2, 1],

                  '路径线': [0., 0.2, 1],

                  '角1': [0.2, 0.2, 1],
                  '角2': [0.2, 0.2, 1],
                  }
    para_dict = {'lenth': 0.0, 'width': 0.0, 'hight': 0.0, 'tumor_volum': 0.0, 'tumor_area': 0.0, 'depth': 0.0,
                 'angle_1': 0.0, 'angle_2': 0.0, 'radius_1': 0.0, 'radius_2': 0.0, 'radius_3': 0.0, 'radius_4': 0.0,
                 'radius_5': 0.0, 'radius_6': 0.0, 'radius_7': 0.0, 'radius_8': 0.0, 'radius_9': 0.0, 'radius_10': 0.0,
                 'radius_11': 0.0}
    window_level = 400
    window_width = 1000
    original_view = vmi.View()
    original_slice = vmi.ImageSlice(original_view)  # 断层显示
    original_slice.setData(original_image)  # 载入读取到的DICOM图像
    original_slice.setColorWindow_Bone()
    original_slice.setSlicePlaneOrigin_Center()
    original_slice.setSlicePlaneNormal_Coronal()
    original_view.setCamera_Coronal()
    original_view.setCamera_FitAll()

    original_slice_menu = vmi.Menu()
    original_slice_menu.menu.addAction('定位目标区域').triggered.connect(on_init_voi)
    original_slice_menu.menu.addSeparator()
    original_slice_menu.menu.addMenu(original_slice.menu)
    original_slice.mouse['RightButton']['PressRelease'] = [original_slice_menu.menuexec]
    original_slice.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
    original_view.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
    original_view.mouse['LeftButton']['PressMoveRelease'] = [LeftButtonPressMoveRelease]

    original_view_text = ['姓名：{}', '标定区域', '窗宽:{}', '窗位:{}']
    original_view_text_format = [patient_name, '', window_level, window_width]
    original_view_textboxes = []
    for i, text in enumerate(original_view_text):
        original_view_textboxes.append(vmi.TextBox(original_view, text=text.format(original_view_text_format[i]),
                                                   size=[0.24, 0.04], pos=[0, (i + 1) * 0.04], anchor=[0, 0],
                                                   pickable=True))
    # 标定目标区域

    standard_pts = []
    standard_prop = vmi.PolyActor(original_view, color=[1, 0.4, 0.4], line_width=3)
    standard_prop.setAlwaysOnTop(True)
    original_view_textboxes[1].mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
    # 调整窗宽窗位
    original_view_textboxes[2].mouse['NoButton']['Wheel'] = [NoButtonWheel]
    original_view_textboxes[3].mouse['NoButton']['Wheel'] = [NoButtonWheel]

    voi_view = vmi.View()
    voi_view.mouse['LeftButton']['Press'] = [LeftButtonPress]
    voi_view.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
    voi_view.mouse['LeftButton']['PressMoveRelease'] = [LeftButtonPressMoveRelease]
    voi_view.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
    threshold, back_value = window_level, -1024

    voi_image = vtk.vtkImageData()
    voi_volume = vmi.ImageVolume(voi_view, pickable=True)
    voi_volume.setOpacity_Threshold(threshold)
    voi_volume.setColor({threshold: [1, 0.4, 0.4]})

    voi_view_textboxes = [vmi.TextBox(voi_view, text='阈值：{} HU'.format(threshold),
                                      size=[0.24, 0.04], pos=[0, 0.04], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='线切割'.format(threshold),
                                      size=[0.24, 0.04], pos=[0, 0.1], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='勾画动脉瘤'.format(threshold),
                                      size=[0.12, 0.04], pos=[0, 0.16], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='勾画载流动脉'.format(threshold),
                                      size=[0.12, 0.04], pos=[0.12, 0.16], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='测量动脉瘤'.format(threshold),
                                      size=[0.12, 0.04], pos=[0, 0.22], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='测量载流动脉'.format(threshold),
                                      size=[0.12, 0.04], pos=[0.12, 0.22], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='路径规划'.format(threshold),
                                      size=[0.12, 0.04], pos=[0, 0.28], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='支撑设计'.format(threshold),
                                      size=[0.12, 0.04], pos=[0.12, 0.28], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='截面测量',
                                      size=[0.12, 0.04], pos=[0, 0.34], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='长度测量',
                                      size=[0.12, 0.04], pos=[0.12, 0.34], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='选择角一',
                                      size=[0.12, 0.04], pos=[0, 0.4], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='选择角二',
                                      size=[0.12, 0.04], pos=[0.12, 0.4], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='隐藏整体数据',
                                      size=[0.12, 0.04], pos=[0, 0.46], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='隐藏动脉瘤',
                                      size=[0.12, 0.04], pos=[0.12, 0.46], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='隐藏载流动脉',
                                      size=[0.12, 0.04], pos=[0, 0.52], anchor=[0, 0],
                                      pickable=True),

                          vmi.TextBox(voi_view, text='隐藏表面点',
                                      size=[0.12, 0.04], pos=[0.12, 0.52], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='隐藏导管路径',
                                      size=[0.12, 0.04], pos=[0, 0.58], anchor=[0, 0],
                                      pickable=True),
                          vmi.TextBox(voi_view, text='隐藏支撑',
                                      size=[0.12, 0.04], pos=[0.12, 0.58], anchor=[0, 0],
                                      pickable=True,
                                      ),
                          vmi.TextBox(voi_view, text='弹簧圈直径:{}in'.format(pring_coil),
                                      size=[0.24, 0.04], pos=[0, 0.64], anchor=[0, 0],
                                      pickable=True,
                                      ),
                          vmi.TextBox(voi_view, text='填充长度:{}mm'.format(filling_lenth),
                                      size=[0.24, 0.04], pos=[0.0, 0.70], anchor=[0, 0],
                                      pickable=True,
                                      ),
                          vmi.TextBox(voi_view, text='填充率:{}%'.format(fill_ratio),
                                      size=[0.24, 0.04], pos=[0.0, 0.76], anchor=[0, 0],
                                      pickable=True,
                                      ),
                          ]
    for text_box in voi_view_textboxes:
        text_box.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]
    voi_view_textboxes[-3].mouse['NoButton']['Wheel'] = [NoButtonWheel]
    voi_view_textboxes[-2].mouse['NoButton']['Wheel'] = [NoButtonWheel]
    voi_view_textboxes[-1].mouse['NoButton']['Wheel'] = [NoButtonWheel]

    para_1_box = vmi.TextBox(voi_view, text='动脉瘤参数:\n长:{}mm\n宽:{}mm\n高:{}mm\n体积:{}mm³'.format(0.00, 0.00, 0.00, 0.00),
                             size=[0.18, 0.15], pos=[0.26, 0.04], anchor=[0, 0])
    para_2_box = vmi.TextBox(voi_view, text='\n截面积:{}.mm²\n瘤颈深度:{}.mm\n角一:{}°\n角二:{}°'.format(0.00, 0.00, 0.00, 0.00),
                             size=[0.18, 0.15], pos=[0.43, 0.04], anchor=[0, 0])
    para_3_box = vmi.TextBox(voi_view,
                             text='载流动脉参数:\n半径1:{}mm\n半径2:{}mm\n半径3:{}mm\n半径4:{}mm\n半径5:{}mm'.format(0.00, 0.00, 0.00,
                                                                                                     0.00, 0.00),
                             size=[0.18, 0.15], pos=[0.62, 0.04], anchor=[0, 0])
    para_4_box = vmi.TextBox(voi_view,
                             text='半径6:{}mm\n半径7:{}mm\n半径8:{}mm\n半径9:{}mm\n半径10:{}mm\n半径11:{}mm'.format(0.00, 0.00,
                                                                                                        0.00, 0.00,
                                                                                                        0.00, 0.00),
                             size=[0.18, 0.15], pos=[0.79, 0.04], anchor=[0, 0])
    plcut_pts = []
    plcut_prop = vmi.PolyActor(voi_view, color=[1, 0.4, 0.4], line_width=3)
    plcut_prop.setAlwaysOnTop(True)
    tumor_pts = []
    tumor_cline_prop = vmi.PolyActor(voi_view, color=[1, 0.4, 0.4], line_width=3)
    tumor_cline_prop.setAlwaysOnTop(True)
    vessel_pts = []
    vessel_cline_prop = vmi.PolyActor(voi_view, color=[1, 0.4, 0.4], line_width=3)
    vessel_cline_prop.setAlwaysOnTop(True)
    # 中心点和中心线 载瘤动脉和动脉瘤
    vessel_f_l, vessel_f_l_props, vessel_line_props, vessel_centers, vessel_center_props = [], [], [], [], []

    tumor_f_l, tumor_f_l_props, tumor_line_props = [], [], []

    tumor_center_prop = vmi.PolyActor(voi_view, color=[0.8, 0.8, 1], pickable=True)
    tumor_center_prop.mouse['LeftButton']['PressMove'] = [LeftButtonPressMove]
    tumor_center_prop.mouse['LeftButton']['PressRelease'] = [LeftButtonPressRelease]

    pipe_pts, pipe_props = [], []
    pipe_cline = vmi.PolyActor(voi_view, color=[0.7, 0.7, 0.7], line_width=3)
    pipe_cline_tube = vtk.vtkTubeFilter()

    brace_pts, brace_props = [], []
    brace_cline = vmi.PolyActor(voi_view, color=[0.9, 0.9, 0.9], line_width=3)
    brace_cline_tube = vtk.vtkTubeFilter()
    # line, line_pts, line_props = 0, [], []
    # line_measure = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)
    angle_1, angle_1_pts, angle_1_props = 0, [], []
    angle_1_line_1 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)
    angle_1_line_2 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)

    angle_2, angle_2_pts, angle_2_props = 0, [], []
    angle_2_line_1 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)
    angle_2_line_2 = vmi.PolyActor(voi_view, color=[0.2, 1, 0.2], line_width=3)

    section_pts = []
    section_prop = vmi.PolyActor(voi_view, color=[1, 0.4, 0.4], line_width=3)

    widget = QWidget()
    layout = QGridLayout(widget)
    layout.addWidget(original_view, 0, 0, 3, 1)
    layout.addWidget(voi_view, 0, 1, 3, 1)

    main = Main()
    main.setWidget(widget)

    vmi.appexec(main)  # 执行主窗口程序
    vmi.appexit()  # 清理并退出程序
